#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2019 Vamsi K Vytla <vamsi.vytla@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform
from litex.build.openocd import OpenOCD

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk200", 0,
        Subsignal("p", Pins("R3"), IOStandard("DIFF_SSTL15")),
        Subsignal("n", Pins("P3"), IOStandard("DIFF_SSTL15"))
    ),
    ("clk156", 0,
        Subsignal("p", Pins("M21"), IOStandard("LVDS_25")),
        Subsignal("n", Pins("M22"), IOStandard("LVDS_25"))
    ),
    ("cpu_reset", 0, Pins("U4"), IOStandard("SSTL15")),

    # Leds
    ("user_led", 0, Pins("M26"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("T24"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("T25"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("R26"), IOStandard("LVCMOS33")),

    # Serial
    ("serial", 0,
        Subsignal("cts", Pins("V19")),
        Subsignal("rts", Pins("W19")),
        Subsignal("tx",  Pins("U19")),
        Subsignal("rx",  Pins("T19")),
        IOStandard("LVCMOS18")
    ),

    # RGMII Ethernet
    ("eth_clocks", 0,
        Subsignal("tx", Pins("U22")),
        Subsignal("rx", Pins("U21")),
        IOStandard("LVCMOS18")
    ),
    ("eth", 0,
        Subsignal("rx_ctl",  Pins("U14")),
        Subsignal("rx_data", Pins("U17 V17 V16 V14")),
        Subsignal("tx_ctl",  Pins("T15")),
        Subsignal("tx_data", Pins("U16 U15 T18 T17")),
        Subsignal("rst_n",   Pins("V18")),
        Subsignal("mdc",     Pins("W18")),
        Subsignal("mdio",    Pins("T14")),
        Misc("SLEW=FAST"),
        Drive(16),
        IOStandard("LVCMOS18"),
    ),

    # DDR3 SDRAM
    ("ddram", 0,
        Subsignal("a", Pins(
            "M4 J3 J1 L4 K5 M7 K1 M6",
            "H1 K3 N7 L5 L7 N6 L3 K2"),
            IOStandard("SSTL15")),
        Subsignal("ba",    Pins("N1 M1 H2"), IOStandard("SSTL15")),
        Subsignal("ras_n", Pins("P1"), IOStandard("SSTL15")),
        Subsignal("cas_n", Pins("T4"), IOStandard("SSTL15")),
        Subsignal("we_n",  Pins("R1"), IOStandard("SSTL15")),
        Subsignal("cs_n",  Pins("T3"), IOStandard("SSTL15")),
        Subsignal("dm", Pins("AC6 AC4 AA3 U7 G1 F3 G5 H9"),
            IOStandard("SSTL15")),
        Subsignal("dq", Pins(
            "AB6 AA8  Y8 AB5 AA5  Y5  Y6  Y7",
            "AF4 AF5 AF3 AE3 AD3 AC3 AB4 AA4",
            "AC2 AB2 AF2 AE2  Y1  Y2 AC1 AB1",
            "Y3   W3  W6  V6  W4  W5  W1  V1",
            "G2   D1  E1  E2  F2  A2  A3  C2",
            "C3   D3  A4  B4  C4  D4  D5  E5",
            "F4   G4  K6  K7  K8  L8  J5  J6",
            "G6   H6  F7  F8  G8  H8  D6  E6"),
            IOStandard("SSTL15"),
            Misc("IN_TERM=UNTUNED_SPLIT_50")),
        Subsignal("dqs_p", Pins("V8 AD5 AD1 V3 C1 B5 J4 H7"),
            IOStandard("DIFF_SSTL15")),
        Subsignal("dqs_n", Pins("W8 AE5 AE1 V2 B1 A5 H4 G7"),
            IOStandard("DIFF_SSTL15")),
        Subsignal("clk_p", Pins("M2"), IOStandard("DIFF_SSTL15")),
        Subsignal("clk_n", Pins("L2"), IOStandard("DIFF_SSTL15")),
        Subsignal("cke",   Pins("P4"), IOStandard("SSTL15")),
        Subsignal("odt",   Pins("R2"), IOStandard("SSTL15")),
        Subsignal("reset_n", Pins("N8"), IOStandard("LVCMOS15")),
        Misc("SLEW=FAST"),
    ),

    # SPIFlash
    ("spiflash", 0,
        Subsignal("cs_n", Pins("P18")),
        #Subsignal("clk",  Pins("")), # Accessed through STARTUPE2
        Subsignal("mosi", Pins("R14")),
        Subsignal("miso", Pins("R15")),
        Subsignal("wp",   Pins("P14")),
        Subsignal("hold", Pins("N14")),
        IOStandard("LVCMOS33"),
    ),
    ("spiflash4x", 0,
        Subsignal("cs_n", Pins("P18")),
        #Subsignal("clk",  Pins("")), # Accessed through STARTUPE2
        Subsignal("dq",   Pins("R14 R15 P14 N14")),
        IOStandard("LVCMOS33")
    ),

    # PCIe
    ("pcie_x1", 0,
        Subsignal("rst_n", Pins("M20"), IOStandard("LVCMOS25")),
        Subsignal("clk_p", Pins("F11")),
        Subsignal("clk_n", Pins("E11")),
        Subsignal("rx_p",  Pins("D12")),
        Subsignal("rx_n",  Pins("C12")),
        Subsignal("tx_p",  Pins("D10")),
        Subsignal("tx_n",  Pins("C10"))
    ),

    ("pcie_x4", 0,
        Subsignal("rst_n", Pins("M20"), IOStandard("LVCMOS25")),
        Subsignal("clk_p", Pins("F11")),
        Subsignal("clk_n", Pins("E11")),
        Subsignal("rx_p",  Pins("D12 B13 D14 B11")),
        Subsignal("rx_n",  Pins("C12 A13 C14 A11")),
        Subsignal("tx_p",  Pins("D10 B9 D8 B7")),
        Subsignal("tx_n",  Pins("C10 A9 C8 A7"))
    ),

    # GTP RefClk
    ("gtp_refclk", 0,
     Subsignal("p", Pins("AA13")),
     Subsignal("n", Pins("AB13"))
    ),

    # SFP
    ("sfp", 0,
        Subsignal("txp", Pins("AC10")),
        Subsignal("txn", Pins("AD10")),
        Subsignal("rxp", Pins("AC12")),
        Subsignal("rxn", Pins("AD12")),
    ),
    ("sfp_mgt_clk_sel0", 0, Pins("B26"), IOStandard("LVCMOS25")),
    ("sfp_mgt_clk_sel1", 0, Pins("C24"), IOStandard("LVCMOS25")),
    ("sfp_tx_disable_n", 0, Pins("R18"), IOStandard("LVCMOS33")),
    ("sfp_rx_los",       0, Pins("R23"), IOStandard("LVCMOS33")),

    # Others
    ("vadj_on_b", 0, Pins("R16"), IOStandard("LVCMOS25")),

]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    ("HPC", {
      "CLK0_M2C_N" : "C19",
      "CLK0_M2C_P" : "D19",
      "CLK1_M2C_N" : "H22",
      "CLK1_M2C_P" : "H21",
      "LA00_CC_N"  : "C18",
      "LA00_CC_P"  : "D18",
      "LA01_CC_N"  : "E18",
      "LA01_CC_P"  : "E17",
      "LA02_N"     : "H15",
      "LA02_P"     : "H14",
      "LA03_N"     : "F17",
      "LA03_P"     : "G17",
      "LA04_N"     : "F19",
      "LA04_P"     : "F18",
      "LA05_N"     : "F15",
      "LA05_P"     : "G15",
      "LA06_N"     : "F20",
      "LA06_P"     : "G19",
      "LA07_N"     : "G16",
      "LA07_P"     : "H16",
      "LA08_N"     : "B17",
      "LA08_P"     : "C17",
      "LA09_N"     : "D16",
      "LA09_P"     : "E16",
      "LA10_N"     : "A18",
      "LA10_P"     : "A17",
      "LA11_N"     : "A19",
      "LA11_P"     : "B19",
      "LA12_N"     : "D20",
      "LA12_P"     : "E20",
      "LA13_N"     : "A20",
      "LA13_P"     : "B20",
      "LA14_N"     : "B21",
      "LA14_P"     : "C21",
      "LA15_N"     : "A22",
      "LA15_P"     : "B22",
      "LA16_N"     : "D21",
      "LA16_P"     : "E21",
      "LA17_CC_N"  : "J21",
      "LA17_CC_P"  : "K21",
      "LA18_CC_N"  : "G21",
      "LA18_CC_P"  : "G20",
      "LA19_N"     : "L14",
      "LA19_P"     : "M14",
      "LA20_N"     : "M17",
      "LA20_P"     : "M16",
      "LA21_N"     : "H19",
      "LA21_P"     : "J19",
      "LA22_N"     : "L18",
      "LA22_P"     : "L17",
      "LA23_N"     : "J20",
      "LA23_P"     : "K20",
      "LA24_N"     : "H18",
      "LA24_P"     : "J18",
      "LA25_N"     : "F22",
      "LA25_P"     : "G22",
      "LA26_N"     : "H24",
      "LA26_P"     : "J24",
      "LA27_N"     : "E23",
      "LA27_P"     : "F23",
      "LA28_N"     : "K23",
      "LA28_P"     : "K22",
      "LA29_N"     : "F24",
      "LA29_P"     : "G24",
      "LA30_N"     : "D25",
      "LA30_P"     : "E25",
      "LA31_N"     : "D26",
      "LA31_P"     : "E26",
      "LA32_N"     : "G26",
      "LA32_P"     : "H26",
      "LA33_N"     : "F25",
      "LA33_P"     : "G25",
      "PRSNT_M2C_L"         : "N16",
      "PWR_GOOD_FLASH_RST_B": "P15"}
    ),
    ("XADC", {
        "GPIO0"   : "H17",
        "GPIO1"   : "E22",
        "GPIO2"   : "K18",
        "GPIO3"   : "L19",
        "VAUX0_N" : "J16",
        "VAUX0_P" : "K15",
        "VAUX8_N" : "J15",
        "VAUX8_P" : "J14",
        }
    ),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk156"
    default_clk_period = 1e9/156.5e6

    def __init__(self, toolchain="vivado"):
        Xilinx7SeriesPlatform.__init__(self, "xc7a200t-fbg676-2", _io, _connectors, toolchain=toolchain)
        self.toolchain.bitstream_commands = ["set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]"]
        self.toolchain.additional_commands = ["write_cfgmem -force -format bin -interface spix4 -size 16 -loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]
        self.add_platform_command("set_property INTERNAL_VREF 0.750 [get_iobanks 33]")
        self.add_platform_command("set_property INTERNAL_VREF 0.750 [get_iobanks 34]")
        self.add_platform_command("set_property INTERNAL_VREF 0.750 [get_iobanks 35]")

    def create_programmer(self):
        return OpenOCD("openocd_xc7_ft2232.cfg", "bscan_spi_xc7a200t.bit")

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk200",        loose=True), 1e9/200e6)
        self.add_period_constraint(self.lookup_request("eth_clocks:rx", loose=True), 1e9/125e6)
        self.add_period_constraint(self.lookup_request("eth_clocks:tx", loose=True), 1e9/125e6)



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 Gwenhael Goavec-Merou <gwenhael.goavec-merou@trabucayre.com>
# SPDX-License-Identifier: BSD-2-Clause

import os

from migen import *

from litex.gen import *

from litex_boards.platforms import redpitaya

from litex.soc.interconnect import axi
from litex.soc.interconnect import wishbone

from litex.soc.cores.clock import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.soc import SoCRegion
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser

# CRG ----------------------------------------------------------------------------------------------


class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq, use_ps7_clk=False):
        self.rst    = Signal()
        self.cd_sys = ClockDomain()

        # # #

        if use_ps7_clk:
            assert sys_clk_freq == 125e6
            self.comb += ClockSignal("sys").eq(ClockSignal("ps7"))
            self.comb += ResetSignal("sys").eq(ResetSignal("ps7") | self.rst)
        else:
            self.pll = pll = S7PLL(speedgrade=-1)
            self.comb += pll.reset.eq(self.rst)
            pll.register_clkin(platform.request(platform.default_clk_name), platform.default_clk_freq)
            pll.create_clkout(self.cd_sys,      sys_clk_freq)
            platform.add_false_path_constraints(self.cd_sys.clk, pll.clkin) # Ignore sys_clk to pll.clkin path created by SoC's rst.

# BaseSoC ------------------------------------------------------------------------------------------


class BaseSoC(SoCCore):
    def __init__(self, board, sys_clk_freq=100e6, with_led_chaser=True, **kwargs):
        platform = redpitaya.Platform(board)

        # CRG --------------------------------------------------------------------------------------
        use_ps7_clk  = (kwargs.get("cpu_type", None) == "zynq7000")
        sys_clk_freq = 125e6 if use_ps7_clk else sys_clk_freq
        self.crg = _CRG(platform, sys_clk_freq, use_ps7_clk)

        # SoCCore ----------------------------------------------------------------------------------
        if kwargs["uart_name"] == "serial":
            kwargs["uart_name"] = "usb_uart"
        if kwargs.get("cpu_type", None) == "zynq7000":
            kwargs["integrated_sram_size"] = 0
            kwargs["with_uart"]            = False
            self.mem_map = {
                'csr': 0x43c0_0000,  # Zynq GP0 default
            }
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on Zebboard", **kwargs)

        # Zynq7000 Integration ---------------------------------------------------------------------
        if kwargs.get("cpu_type", None) == "zynq7000":
            # Get and set the pre-generated .xci FIXME: change location? add it to the repository?
            os.system("wget https://kmf2.trabucayre.com/redpitaya_ps7.txt")
            os.makedirs("xci", exist_ok=True)
            os.system("cp redpitaya_ps7.txt xci/redpitaya_ps7.xci")
            self.cpu.set_ps7_xci("xci/redpitaya_ps7.xci")

            # Connect AXI GP0 to the SoC with base address of 0x43c00000 (default one)
            wb_gp0  = wishbone.Interface()
            self.submodules += axi.AXI2Wishbone(
                axi          = self.cpu.add_axi_gp_master(),
                wishbone     = wb_gp0,
                base_address = 0x43c00000)
            self.bus.add_master(master=wb_gp0)
            self.bus.add_region("flash",  SoCRegion(origin=0xFC00_0000, size=0x4_0000, mode="rwx"))

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------


def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=redpitaya.Platform, description="LiteX SoC on Zedboard.")
    parser.add_target_argument("--sys-clk-freq", default=100e6, type=float, help="System clock frequency.")
    parser.add_target_argument("--board",        default="redpitaya14",     help="Board type (redpitaya14 or redpitaya16).")
    args = parser.parse_args()

    soc = BaseSoC(
        board        = args.board,
        sys_clk_freq = args.sys_clk_freq,
        **parser.soc_argdict
    )
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"), device=1)

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 Fei Gao <feig@princeton.edu>
# Copyright (c) 2020 Florent Kermarrec <florent@enjoy-digital.fr>
# Copyright (c) 2022 Jiajie Chen <c@jia.je>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import XilinxUSPPlatform, VivadoProgrammer

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk100_ddr4", 0,
        Subsignal("p", Pins("BH51"), IOStandard("LVDS")),
        Subsignal("n", Pins("BJ51"), IOStandard("LVDS")),
    ),
    ("clk100_qdr4", 0,
        Subsignal("p", Pins("BJ4"), IOStandard("LVDS")),
        Subsignal("n", Pins("BK3"), IOStandard("LVDS")),
    ),
    ("clk100_rld3", 0,
        Subsignal("p", Pins("F35"), IOStandard("LVDS")),
        Subsignal("n", Pins("F36"), IOStandard("LVDS")),
    ),
    ("cpu_reset", 0, Pins("BM29"), IOStandard("LVCMOS12")),

    # Leds
    ("user_led", 0, Pins("BH24"), IOStandard("LVCMOS18")),
    ("user_led", 1, Pins("BG24"), IOStandard("LVCMOS18")),
    ("user_led", 2, Pins("BG25"), IOStandard("LVCMOS18")),
    ("user_led", 3, Pins("BF25"), IOStandard("LVCMOS18")),
    ("user_led", 4, Pins("BF26"), IOStandard("LVCMOS18")),
    ("user_led", 5, Pins("BF27"), IOStandard("LVCMOS18")),
    ("user_led", 6, Pins("BG27"), IOStandard("LVCMOS18")),
    ("user_led", 7, Pins("BG28"), IOStandard("LVCMOS18")),

    # Serial
    ("serial", 0,
        Subsignal("rx",  Pins("BP26"), IOStandard("LVCMOS18")),
        Subsignal("rts", Pins("BP22"), IOStandard("LVCMOS18")),
        Subsignal("tx",  Pins("BN26"), IOStandard("LVCMOS18")),
        Subsignal("cts", Pins("BP23"), IOStandard("LVCMOS18")),
    ),
    ("serial", 1,
        Subsignal("rx",  Pins("BK28"), IOStandard("LVCMOS18")),
        Subsignal("rts", Pins("BL26"), IOStandard("LVCMOS18")),
        Subsignal("tx",  Pins("BJ28"), IOStandard("LVCMOS18")),
        Subsignal("cts", Pins("BL27"), IOStandard("LVCMOS18")),
    ),

    # DDR4 memory
    ("ddram", 0,
        Subsignal("a", Pins(
            "BF50 BD51 BG48 BE50 BE49 BE51 BF53 BG50",
            "BF51 BG47 BF47 BG49 BF48 BF52"),
            IOStandard("SSTL12_DCI")),
        Subsignal("ba",        Pins("BE54 BE53"), IOStandard("SSTL12_DCI")),
        Subsignal("bg",        Pins("BG54"), IOStandard("SSTL12_DCI")),
        Subsignal("ras_n",     Pins("BJ54"), IOStandard("SSTL12_DCI")),
        Subsignal("cas_n",     Pins("BH54"), IOStandard("SSTL12_DCI")),
        Subsignal("we_n",      Pins("BG53"), IOStandard("SSTL12_DCI")),
        Subsignal("cs_n",      Pins("BP49 BK48"), IOStandard("SSTL12_DCI")), # Clam-shell topology
        Subsignal("act_n",     Pins("BG52"), IOStandard("SSTL12_DCI")),
        #Subsignal("ten",       Pins("BJ53"), IOStandard("SSTL12_DCI")),
        #Subsignal("alert_n",   Pins("BJ52"), IOStandard("SSTL12_DCI")),
        #Subsignal("par",       Pins("BL48"), IOStandard("SSTL12_DCI")),
        Subsignal("dm",        Pins(
            "BN42 BL47 BH42 BD41 BM28 BM34 BH32 BG29"),
            IOStandard("POD12_DCI")),
        Subsignal("dq",        Pins(
            "BM45 BP44 BP47 BN45 BM44 BN44 BN47 BP43",
            "BL45 BK44 BL46 BK43 BL43 BJ44 BL42 BJ43",
            "BK41 BG44 BG42 BH44 BH45 BG45 BG43 BJ41",
            "BE43 BF42 BC42 BF43 BD42 BF45 BE44 BF46",
            "BP32 BP29 BP31 BP28 BN32 BM30 BN31 BL30",
            "BL32 BP34 BN34 BK33 BL31 BL33 BM33 BK31",
            "BJ34 BG35 BH34 BH35 BJ33 BF35 BG34 BF36",
            "BF31 BH30 BJ31 BG32 BH31 BF32 BH29 BF33"),
            IOStandard("POD12_DCI"),
            Misc("PRE_EMPHASIS=RDRV_240"),
            Misc("EQUALIZATION=EQ_LEVEL2")),
        Subsignal("dqs_p",   Pins("BN46 BK45 BH46 BE45 BN29 BL35 BK34 BJ29"),
		    IOStandard("DIFF_POD12"),
            Misc("PRE_EMPHASIS=RDRV_240"),
            Misc("EQUALIZATION=EQ_LEVEL2")),
        Subsignal("dqs_n",   Pins("BP46 BK46 BJ46 BE46 BN30 BM35 BK35 BK30"),
		    IOStandard("DIFF_POD12"),
            Misc("PRE_EMPHASIS=RDRV_240"),
            Misc("EQUALIZATION=EQ_LEVEL2")),
        Subsignal("clk_p",   Pins("BK53"), IOStandard("DIFF_SSTL12_DCI")),
        Subsignal("clk_n",   Pins("BK54"), IOStandard("DIFF_SSTL12_DCI")),
        Subsignal("cke",     Pins("BH52"), IOStandard("SSTL12_DCI")),
        Subsignal("odt",     Pins("BH49"), IOStandard("SSTL12_DCI")),
        Subsignal("reset_n", Pins("BH50"), IOStandard("LVCMOS12")),
        Misc("SLEW=FAST"),
    ),

    # SGMII Clock
    ("eth_clocks", 0,
        Subsignal("p", Pins("BH27"), IOStandard("LVDS")),
        Subsignal("n", Pins("BJ27"), IOStandard("LVDS")),
    ),

    # SGMII Ethernet
    ("eth", 0,
        Subsignal("int_n", Pins("BF22"), IOStandard("LVCMOS18")),
        Subsignal("mdio",  Pins("BG23"), IOStandard("LVCMOS18")),
        Subsignal("mdc",   Pins("BN27"), IOStandard("LVCMOS18")),
        Subsignal("rx_p",  Pins("BJ22"), IOStandard("LVDS")),
        Subsignal("rx_n",  Pins("BK21"), IOStandard("LVDS")),
        Subsignal("tx_p",  Pins("BG22"), IOStandard("LVDS")),
        Subsignal("tx_n",  Pins("BH22"), IOStandard("LVDS")),
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = []

# Platform -----------------------------------------------------------------------------------------

class Platform(XilinxUSPPlatform):
    default_clk_name   = "clk100_ddr4"
    default_clk_period = 1e9/100e6

    def __init__(self, toolchain="vivado"):
        XilinxUSPPlatform.__init__(self, "xcvu37p-fsvh2892-2L-e", _io, _connectors, toolchain="vivado")

    def create_programmer(self):
        return VivadoProgrammer()

    def do_finalize(self, fragment):
        XilinxUSPPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk100_ddr4", loose=True), 1e9/100e6)
        self.add_period_constraint(self.lookup_request("clk100_qdr4", loose=True), 1e9/100e6)
        self.add_period_constraint(self.lookup_request("clk100_rld3", loose=True), 1e9/100e6)

        # DDR4 memory Internal Vref
        self.add_platform_command("set_property INTERNAL_VREF 0.84 [get_iobanks 64]")
        self.add_platform_command("set_property INTERNAL_VREF 0.84 [get_iobanks 65]")
        self.add_platform_command("set_property INTERNAL_VREF 0.84 [get_iobanks 66]")

        # For HBM2 IP in Vivado 2019.2 (https://www.xilinx.com/support/answers/72607.html)
        self.add_platform_command("connect_debug_port dbg_hub/clk [get_nets apb_clk]")



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 Fei Gao <feig@princeton.edu>
# Copyright (c) 2020 Florent Kermarrec <florent@enjoy-digital.fr>
# Copyright (c) 2022 Jiajie Chen <c@jia.je>
# SPDX-License-Identifier: BSD-2-Clause

from migen import *
from migen.genlib.resetsync import AsyncResetSynchronizer

from litex.gen import *

from litex_boards.platforms import xilinx_vcu128

from litex.soc.cores.clock import *
from litex.soc.cores.ram.xilinx_usp_hbm2 import USPHBM2
from litex.soc.interconnect.axi import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.soc import SoCRegion
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser

from litedram.modules import MT40A512M16
from litedram.phy import usddrphy

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq, with_hbm):
        self.rst       = Signal()
        self.cd_sys    = ClockDomain()
        if with_hbm:
            self.cd_hbm_ref = ClockDomain()
            self.cd_apb     = ClockDomain()
        else: # DDR4
            self.cd_sys4x  = ClockDomain()
            self.cd_pll4x  = ClockDomain()
            self.cd_idelay = ClockDomain()

        # # #

        self.pll = pll = USMMCM(speedgrade=-2)
        self.comb += pll.reset.eq(platform.request("cpu_reset") | self.rst)
        pll.register_clkin(platform.request("clk100_ddr4"), 100e6)
        if with_hbm:
            pll.create_clkout(self.cd_sys,     sys_clk_freq)
            pll.create_clkout(self.cd_hbm_ref, 100e6)
            pll.create_clkout(self.cd_apb,     100e6)
            platform.add_false_path_constraints(self.cd_sys.clk, self.cd_apb.clk)
        else:
            pll.create_clkout(self.cd_pll4x, sys_clk_freq*4, buf=None, with_reset=False)
            pll.create_clkout(self.cd_idelay, 500e6)
            platform.add_false_path_constraints(self.cd_sys.clk, pll.clkin) # Ignore sys_clk to pll.clkin path created by SoC's rst.

            self.specials += [
                Instance("BUFGCE_DIV",
                    p_BUFGCE_DIVIDE=4,
                    i_CE=1, i_I=self.cd_pll4x.clk, o_O=self.cd_sys.clk),
                Instance("BUFGCE",
                    i_CE=1, i_I=self.cd_pll4x.clk, o_O=self.cd_sys4x.clk),
            ]

            self.idelayctrl = USIDELAYCTRL(cd_ref=self.cd_idelay, cd_sys=self.cd_sys)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, sys_clk_freq=125e6, with_led_chaser=True, with_hbm=False, **kwargs):
        platform = xilinx_vcu128.Platform()

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq, with_hbm)

        # SoCCore ----------------------------------------------------------------------------------
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on VCU128", **kwargs)

        # HBM --------------------------------------------------------------------------------------
        if with_hbm:
            # Add HBM Core.
            self.hbm = hbm = ClockDomainsRenamer({"axi": "sys"})(USPHBM2(platform))

            # Get HBM .xci.
            os.system("wget https://github.com/litex-hub/litex-boards/files/6893157/hbm_0.xci.txt")
            os.makedirs("ip/hbm", exist_ok=True)
            os.system("mv hbm_0.xci.txt ip/hbm/hbm_0.xci")

            # Connect four of the HBM's AXI interfaces to the main bus of the SoC.
            for i in range(4):
                axi_hbm      = hbm.axi[i]
                axi_lite_hbm = AXILiteInterface(data_width=256, address_width=33)
                self.submodules += AXILite2AXI(axi_lite_hbm, axi_hbm)
                self.bus.add_slave(f"hbm{i}", axi_lite_hbm, SoCRegion(origin=0x4000_0000 + 0x1000_0000*i, size=0x1000_0000)) # 256MB.
            # Link HBM2 channel 0 as main RAM
            self.bus.add_region("main_ram", SoCRegion(origin=0x4000_0000, size=0x1000_0000, linker=True)) # 256MB.
        elif not self.integrated_main_ram_size:
            # DDR4 SDRAM -------------------------------------------------------------------------------
            self.ddrphy = usddrphy.USPDDRPHY(
                pads             = platform.request("ddram"),
                memtype          = "DDR4",
                sys_clk_freq     = sys_clk_freq,
                is_clam_shell    = True,
                iodelay_clk_freq = 500e6)
            self.add_sdram("sdram",
                phy           = self.ddrphy,
                module        = MT40A512M16(sys_clk_freq, "1:4"),
                size          = 0x40000000,
                l2_cache_size = kwargs.get("l2_size", 8192)
            )

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=xilinx_vcu128.Platform, description="LiteX SoC on VCU128.")
    parser.add_target_argument("--sys-clk-freq",  default=125e6, type=float, help="System clock frequency.")
    parser.add_target_argument("--with-hbm",      action="store_true",       help="Use HBM2.")
    args = parser.parse_args()

    soc = BaseSoC(
        sys_clk_freq = args.sys_clk_freq,
        with_hbm     = args.with_hbm,
        **parser.soc_argdict
    )
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2019-2021 Antti Lukats <antti.lukats@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause
#
# http://trenz.org/max1000-info

from litex.build.generic_platform import *
from litex.build.altera import AlteraPlatform
from litex.build.altera.programmer import USBBlaster

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk12", 0, Pins("H6"), IOStandard("3.3-V LVTTL")),

    # Leds
    ("user_led", 0, Pins("A8"),  IOStandard("3.3-V LVTTL")),
    ("user_led", 1, Pins("A9"),  IOStandard("3.3-V LVTTL")),
    ("user_led", 2, Pins("A11"), IOStandard("3.3-V LVTTL")),
    ("user_led", 3, Pins("A10"), IOStandard("3.3-V LVTTL")),
    ("user_led", 4, Pins("B10"), IOStandard("3.3-V LVTTL")),
    ("user_led", 5, Pins("C9"),  IOStandard("3.3-V LVTTL")),
    ("user_led", 6, Pins("C10"), IOStandard("3.3-V LVTTL")),
    ("user_led", 7, Pins("D8"),  IOStandard("3.3-V LVTTL")),

    # Buttons
    ("user_btn", 0, Pins("E6"), IOStandard("3.3-V LVTTL")),
    ("user_btn", 1, Pins("E7"), IOStandard("3.3-V LVTTL")), # nConfig.

    # Serial
    ("serial", 0,
        Subsignal("tx", Pins("B4"), IOStandard("3.3-V LVTTL")),
        Subsignal("rx", Pins("A4"), IOStandard("3.3-V LVTTL"))
    ),

    # SPI Flash
    ("spiflash4x", 0,
        Subsignal("cs_n", Pins("B3")),
        Subsignal("clk", Pins("A3")),
        Subsignal("dq", Pins("A2", "B2", "B9", "C4")),
        IOStandard("3.3-V LVTTL")
    ),
    ("spiflash", 0,
        Subsignal("cs_n", Pins("B3")),
        Subsignal("clk", Pins("A3")),
        Subsignal("mosi", Pins("A2")),
        Subsignal("miso", Pins("B2")),
        Subsignal("wp", Pins("B9")),
        Subsignal("hold", Pins("C4")),
        IOStandard("3.3-V LVTTL"),
    ),

    # SDRAM
    ("sdram_clock", 0, Pins("M9"), IOStandard("3.3-V LVTTL")),
    ("sdram", 0,
        Subsignal("a", Pins(
            "K6  M5 N5 J8 N10 M11 N9 L10",
            "M13 N8 N4 M10")),
        Subsignal("ba",    Pins("N6 K8")),
        Subsignal("cs_n",  Pins("M4")),
        Subsignal("cke",   Pins("M8")),
        Subsignal("ras_n", Pins("M7")),
        Subsignal("cas_n", Pins("N7")),
        Subsignal("we_n",  Pins("K7")),
        Subsignal("dq", Pins(
            "D11 G10 F10  F9 E10  D9  G9  F8",
            "F13 E12 E13 D12 C12 B12 B13 A12")),
        Subsignal("dm", Pins("E9 F12")),
        IOStandard("3.3-V LVTTL")
    ),

    # all IO not connected to peripherals mapped to MFIO
    #                 <-        LEDS           -> <-         PMOD      -> <-                     D0..D14, D11R, D12R                  -> <-     AIN0..AIN7    -> JE [C O  I  S  i1 i2]sw
    ("bbio", 0, Pins("A8 A9 A11 A10 B10 C9 C10 D8 M3 L3 M2 M1 N3 N2 K2 K1 H8 K10 H5 H4 J1 J2 L12 J12 J13 K11 K12 J10 H10 H13 G12 B11 G13 E1 C2 C1 D1 E3 F1 E4 B1 E5 J6 J7 K5 L5 J5 L4 E6"),
        IOStandard("3.3-V LVTTL")),


]

# Platform -----------------------------------------------------------------------------------------

class Platform(AlteraPlatform):
    default_clk_name   = "clk12"
    default_clk_period = 1e9/12e6

    def __init__(self, toolchain="quartus"):
        AlteraPlatform.__init__(self, "10M08SAU169C8G", _io, toolchain=toolchain)
        self.add_platform_command("set_global_assignment -name FAMILY \"MAX 10\"")
        self.add_platform_command("set_global_assignment -name ENABLE_CONFIGURATION_PINS OFF")
        self.add_platform_command("set_global_assignment -name INTERNAL_FLASH_UPDATE_MODE \"SINGLE IMAGE WITH ERAM\"")

    def create_programmer(self):
        return USBBlaster()

    def do_finalize(self, fragment):
        AlteraPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk12", loose=True), 1e9/12e6)
        # Generate PLL clock in STA
        self.toolchain.additional_sdc_commands.append("derive_pll_clocks")
        # Calculates clock uncertainties
        self.toolchain.additional_sdc_commands.append("derive_clock_uncertainty")






#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2019 Sean Cross <sean@xobs.io>
# Copyright (c) 2018 David Shah <dave@ds0.me>
# Copyright (c) 2020 Piotr Esden-Tempski <piotr@esden.net>
# Copyright (c) 2020 Florent Kermarrec <florent@enjoy-digital.fr>
# Copyright (c) 2021 Sylvain Munaut <tnt@246tNt.com>
# SPDX-License-Identifier: BSD-2-Clause

# This target file provides a minimal LiteX SoC for the iCEBreaker-bitsy with a CPU,
# its ROM (in SPI Flash), its SRAM, close to the others LiteX targets.
# For more complete example of LiteX SoC for the iCEBreaker-bitsy with more features and
# documentation can be found, refer to :
# https://github.com/icebreaker-fpga/icebreaker-litex-examples

from migen import *
from migen.genlib.resetsync import AsyncResetSynchronizer

from litex.gen import *

from litex_boards.platforms import icebreaker_bitsy

from litex.soc.cores.ram import Up5kSPRAM
from litex.soc.cores.clock import iCE40PLL
from litex.soc.integration.soc_core import *
from litex.soc.integration.soc import SoCRegion
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser


kB = 1024
mB = 1024*kB

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq=48e6, with_usb_pll=False):
        assert not with_usb_pll or sys_clk_freq == 48e6

        self.rst    = Signal()
        self.cd_sys = ClockDomain()
        self.cd_por = ClockDomain()

        # # #

        # Clk/Rst
        clk12 = platform.request("clk12")
        rst_n = platform.request("user_btn_n")

        # Power On Reset
        por_count = Signal(16, reset=2**16-1)
        por_done  = Signal()
        self.comb += self.cd_por.clk.eq(ClockSignal())
        self.comb += por_done.eq(por_count == 0)
        self.sync.por += If(~por_done, por_count.eq(por_count - 1))

        # PLL
        if with_usb_pll:
            self.cd_usb_12 = ClockDomain()
            self.cd_usb_48 = ClockDomain()
            locked = Signal()
            self.specials.pll = pll = Instance("SB_PLL40_2F_PAD",
                i_PACKAGEPIN    = clk12,
                i_RESETB        = rst_n,
                i_BYPASS        = C(0),

                # o_PLLOUTGLOBALA   = self.cd_sys.clk,
                o_PLLOUTGLOBALA   = self.cd_usb_48.clk,
                o_PLLOUTGLOBALB   = self.cd_usb_12.clk,
                o_LOCK            = locked,

                # Create a 48 MHz PLL clock...
                p_FEEDBACK_PATH = "SIMPLE",
                p_PLLOUT_SELECT_PORTA = "GENCLK",
                p_PLLOUT_SELECT_PORTB = "SHIFTREG_0deg",
                p_DIVR          = 0,
                p_DIVF          = 63,
                p_DIVQ          = 4,
                p_FILTER_RANGE  = 1,
            )
            self.specials += AsyncResetSynchronizer(self.cd_sys, ~por_done | ~locked)
            platform.add_period_constraint(self.cd_sys.clk, 48e6)
            platform.add_period_constraint(self.cd_usb_48.clk, 48e6)
            platform.add_period_constraint(self.cd_usb_12.clk, 12e6)
            self.comb += [
                self.cd_sys.clk.eq(self.cd_usb_48.clk),
            ]
        else:
            self.pll = pll = iCE40PLL(primitive="SB_PLL40_PAD")
            self.comb += pll.reset.eq(~rst_n) # FIXME: Add proper iCE40PLL reset support and add back | self.rst.
            pll.register_clkin(clk12, 12e6)
            pll.create_clkout(self.cd_sys, sys_clk_freq, with_reset=False)
            self.specials += AsyncResetSynchronizer(self.cd_sys, ~por_done | ~pll.locked)
            platform.add_period_constraint(self.cd_sys.clk, 1e9/sys_clk_freq)


# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, bios_flash_offset, sys_clk_freq=24e6, revision="v1", with_led_chaser=True, **kwargs):
        platform = icebreaker_bitsy.Platform(revision=revision)

        # CRG --------------------------------------------------------------------------------------
        with_usb_acm = kwargs["uart_name"] == "usb_acm"
        if with_usb_acm:
            sys_clk_freq = 48e6
        self.crg = _CRG(platform, sys_clk_freq, with_usb_pll=with_usb_acm)

        # SoCCore ----------------------------------------------------------------------------------
        # Disable Integrated ROM/SRAM since too large for iCE40 and UP5K has specific SPRAM.
        kwargs["integrated_sram_size"] = 0
        kwargs["integrated_rom_size"]  = 0
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on iCEBreaker-bitsy", **kwargs)

        # 128KB SPRAM (used as 64kB SRAM / 64kB RAM) -----------------------------------------------
        self.spram = Up5kSPRAM(size=128*kB)
        self.bus.add_slave("psram", self.spram.bus, SoCRegion(size=128*kB))
        self.bus.add_region("sram", SoCRegion(
                origin = self.bus.regions["psram"].origin + 0*kB,
                size   = 64*kB,
                linker = True)
        )
        if not self.integrated_main_ram_size:
            self.bus.add_region("main_ram", SoCRegion(
                origin = self.bus.regions["psram"].origin + 64*kB,
                size   = 64*kB,
                linker = True)
            )

        # LED Chaser --------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led_n"),
                sys_clk_freq = sys_clk_freq,
                polarity     = 1)

        # SPI Flash --------------------------------------------------------------------------------
        from litespi.modules import W25Q128JV
        from litespi.opcodes import SpiNorFlashOpCodes as Codes
        self.add_spi_flash(mode="4x", module=W25Q128JV(Codes.READ_1_1_4), with_master=False)

        # Add ROM linker region --------------------------------------------------------------------
        self.bus.add_region("rom", SoCRegion(
            origin = self.bus.regions["spiflash"].origin + bios_flash_offset,
            size   = 32*kB,
            linker = True)
        )
        self.cpu.set_reset_address(self.bus.regions["rom"].origin)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=icebreaker_bitsy.Platform, description="LiteX SoC on iCEBreaker.")
    parser.add_target_argument("--flash",             action="store_true",      help="Flash bitstream and BIOS.")
    parser.add_target_argument("--sys-clk-freq",      default=24e6, type=float, help="System clock frequency.")
    parser.add_target_argument("--bios-flash-offset", default="0xa0000",        help="BIOS offset in SPI Flash.")
    parser.add_target_argument("--revision",          default="v1",             help="Board revision (v0 or v1).")
    args = parser.parse_args()

    soc = BaseSoC(
        bios_flash_offset   = int(args.bios_flash_offset, 0),
        sys_clk_freq        = args.sys_clk_freq,
		revision            = args.revision,
        **parser.soc_argdict
    )
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.flash:
        from litex.build.dfu import DFUProg
        prog_gw = DFUProg(vid="1d50", pid="0x6146", alt=0)
        prog_sw = DFUProg(vid="1d50", pid="0x6146", alt=1)

        prog_gw.load_bitstream(builder.get_bitstream_filename(mode="sram", ext=".bin"), reset=False) # FIXME
        prog_sw.load_bitstream(builder.get_bios_filename())

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Icenowy Zheng <icenowy@aosc.io>
# Copyright (c) 2015 Yann Sionneau <yann.sionneau@gmail.com>
# Copyright (c) 2015-2019 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform, VivadoProgrammer
from litex.build.openocd import OpenOCD

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk200", 0,
        Subsignal("p", Pins("E5"), IOStandard("LVDS_25")),
        Subsignal("n", Pins("D5"), IOStandard("LVDS_25")),
    ),

    # Leds
    ("user_led", 0, Pins("A22"),  IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("C22"),  IOStandard("LVCMOS33")),

    # Switches
    ("sw", 0, Pins("C23"), IOStandard("LVCMOS33")),
    ("sw", 1, Pins("B25"), IOStandard("LVCMOS33")),
    ("sw", 2, Pins("A25"), IOStandard("LVCMOS33")),
    ("sw", 3, Pins("A23"), IOStandard("LVCMOS33")),

    # Serial
    ("serial", 0,
        Subsignal("tx", Pins("G24")),
        Subsignal("rx", Pins("F24")),
        IOStandard("LVCMOS33")
    ),

    # SPIFlash
    ("spiflash", 0,
        Subsignal("cs_n", Pins("P18")),
        Subsignal("clk",  Pins("H13")),
        Subsignal("mosi", Pins("R14")),
        Subsignal("miso", Pins("R15")),
        Subsignal("wp",   Pins("P14")),
        Subsignal("hold", Pins("N14")),
        IOStandard("LVCMOS33"),
    ),
    ("spiflash4x", 0,
        Subsignal("cs_n", Pins("P18")),
        Subsignal("clk",  Pins("H13")),
        Subsignal("dq",   Pins("R14 R15 P14 N14")),
        IOStandard("LVCMOS33")
    ),

    # DDR3 SDRAM
    ("ddram", 0,
        Subsignal("a", Pins(
            "D4 D3 C2 E3 B2 A2 A5 A4 B5 C3",
            "F2 A3 D1 C4 B4"),
            IOStandard("SSTL15")),
        Subsignal("ba",    Pins("G4 F4 F3"), IOStandard("SSTL15")),
        Subsignal("ras_n", Pins("G2"),  IOStandard("SSTL15")),
        Subsignal("cas_n", Pins("G1"),  IOStandard("SSTL15")),
        Subsignal("we_n",  Pins("E2"),  IOStandard("SSTL15")),
        Subsignal("cs_n",  Pins("E1"),  IOStandard("SSTL15")),
        Subsignal("dm", Pins(
            "U2 T5",
            "L7 K1"
            ), IOStandard("SSTL15")),
        Subsignal("dq", Pins(
            "T2 T4 R2 P4 P3 N4 R3 T3",
            "P8 R6 T8 R5 R8 P6 T7 R7",
            "L5 M7 K3 N6 J3 N7 K5 M6",
            "L3 L2 K2 H2 M2 H1 N3 J1"
            ),
            IOStandard("SSTL15"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("dqs_p", Pins(
            "R1 U6",
            "M4 N1"
            ),
            IOStandard("DIFF_SSTL15"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("dqs_n", Pins(
            "P1 U5",
            "L4 M1"
            ),
            IOStandard("DIFF_SSTL15"),
            Misc("IN_TERM=UNTUNED_SPLIT_40")),
        Subsignal("clk_p", Pins("C1"), IOStandard("DIFF_SSTL15")),
        Subsignal("clk_n", Pins("B1"), IOStandard("DIFF_SSTL15")),
        Subsignal("cke",   Pins("J4"), IOStandard("SSTL15")),
        Subsignal("odt",   Pins("H4"), IOStandard("SSTL15")),
        Subsignal("reset_n", Pins("N2"), IOStandard("SSTL15")),
        Misc("SLEW=FAST"),
    ),

    ## sdcard connector
    ("spisdcard", 0,
        Subsignal("clk",  Pins("P24")),
        Subsignal("mosi", Pins("R25"), Misc("PULLUP True")),
        Subsignal("cs_n", Pins("P25"), Misc("PULLUP True")),
        Subsignal("miso", Pins("N23"), Misc("PULLUP True")),
        Misc("SLEW=FAST"),
        IOStandard("LVCMOS33"),
    ),
    ("sdcard", 0,
        Subsignal("data", Pins("N23 M25 N26 P25"), Misc("PULLUP True")),
        Subsignal("cmd",  Pins("R25"), Misc("PULLUP True")),
        Subsignal("clk",  Pins("P24")),
        Subsignal("cd",   Pins("M26"), Misc("PULLUP True")),
        Misc("SLEW=FAST"),
        IOStandard("LVCMOS33"),
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = []

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk200"
    default_clk_period = 1e9/200e6

    def __init__(self, toolchain="vivado"):
        Xilinx7SeriesPlatform.__init__(self, "xc7a100tfgg676-2", _io, _connectors, toolchain=toolchain)
        self.toolchain.bitstream_commands = \
            ["set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 1 [current_design]"]
        self.toolchain.additional_commands = \
            ["write_cfgmem -force -format bin -interface spix1 -size 16 "
             "-loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]

    def create_programmer(self, name='vivado'):
        if name == 'vivado':
            return VivadoProgrammer()
        elif name == 'openocd':
            bscan_spi = "bscan_spi_xc7a100t.bit"
            return OpenOCD("openocd_xc7_ft2232.cfg", bscan_spi)

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk200:p", loose=True), 1e9/200e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2022 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform
from litex.build.openocd import OpenOCD

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk.
    ("sma_clk_in",  0, Pins("N11"), IOStandard("LVCMOS33")),
    ("sma_clk_out", 0, Pins("T13"), IOStandard("LVCMOS33")),

    # Leds.
    ("user_led", 0, Pins("T2"),  IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("T3"),  IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("T4"),  IOStandard("LVCMOS33")),

    # Buttons.
    ("user_btn", 0, Pins("R1"), IOStandard("LVCMOS33")),

    # Switches.
    ("user_sw", 0, Pins("J16"), IOStandard("LVCMOS33")),
    ("user_sw", 1, Pins("K16"), IOStandard("LVCMOS33")),
    ("user_sw", 2, Pins("K15"), IOStandard("LVCMOS33")),
    ("user_sw", 3, Pins("L14"), IOStandard("LVCMOS33")),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    ["expansion",
        # -------------------------------------------------
        "---", # 0
        # -------------------------------------------------
        # 3V3 3V3 GND GND
        " --- --- --- --- A12 B12 A14 A13 A15 B15 C12 C11 B14 C14 B16 C16 C13 D13",
        # 3V3 3V3 GND GND                                         TRI CLK GND GND
        " --- --- --- --- D15 --- E15 D16 E13 E16 F15 F12 E11 F13 --- --- --- ---",
    ],
]

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    def __init__(self, toolchain="vivado"):
        Xilinx7SeriesPlatform.__init__(self, "xc7a100t-ftg256-2", _io, _connectors, toolchain=toolchain)

    def create_programmer(self):
        return OpenOCD("openocd_xc7_ft232.cfg", "bscan_spi_xc7a100t.bit")

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2016-2022 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from migen import *

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk.
    ("clk100", 0, Pins("R4"), IOStandard("LVCMOS33")),

    # Leds.
    ("user_led", 0, Pins("AB1"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("AB8"), IOStandard("LVCMOS33")),

    # Buttons.
    ("user_btn", 0, Pins("AA1"), IOStandard("LVCMOS33")),
    ("user_btn", 1, Pins("AB6"), IOStandard("LVCMOS33")),

    # Serial.
    ("serial", 0,
        Subsignal("tx", Pins("T1")),
        Subsignal("rx", Pins("U1")),
        IOStandard("LVCMOS33"),
    ),

    # DDR3 SDRAM.
    ("ddram", 0,
        Subsignal("a", Pins(
            "M2 M5 M3 M1 L6 P1 N3 N2",
            "M6 R1 L5 N5 N4 P2 P6"),
            IOStandard("SSTL15")),
        Subsignal("ba",    Pins("L3 K6 L4"), IOStandard("SSTL15")),
        Subsignal("ras_n", Pins("J4"),       IOStandard("SSTL15")),
        Subsignal("cas_n", Pins("K3"),       IOStandard("SSTL15")),
        Subsignal("we_n",  Pins("L1"),       IOStandard("SSTL15")),
        Subsignal("dm",    Pins("G3 F1"),    IOStandard("SSTL15")),
        Subsignal("dq", Pins(
            "G2 H4 H5 J1 K1 H3 H2 J5",
            "E3 B2 F3 D2 C2 A1 E2 B1"),
            IOStandard("SSTL15"),
            Misc("IN_TERM=UNTUNED_SPLIT_50")),
        Subsignal("dqs_p",   Pins("K2 E1"), IOStandard("DIFF_SSTL15")),
        Subsignal("dqs_n",   Pins("J2 D1"), IOStandard("DIFF_SSTL15")),
        Subsignal("clk_p",   Pins("P5"),    IOStandard("DIFF_SSTL15")),
        Subsignal("clk_n",   Pins("P4"),    IOStandard("DIFF_SSTL15")),
        Subsignal("cke",     Pins("J6"),    IOStandard("SSTL15")),
        Subsignal("odt",     Pins("K4"),    IOStandard("SSTL15")),
        Subsignal("reset_n", Pins("G1"),    IOStandard("SSTL15")),
        Misc("SLEW=FAST"),
    ),

    # PCIe.
    ("pcie_x1", 0,
        Subsignal("rst_n", Pins("AB7"), IOStandard("LVCMOS33")),
        Subsignal("clk_p", Pins("F6")),
        Subsignal("clk_n", Pins("E6")),
        Subsignal("rx_p",  Pins("B10")),
        Subsignal("rx_n",  Pins("A10")),
        Subsignal("tx_p",  Pins("B6")),
        Subsignal("tx_n",  Pins("A6"))
    ),

    # USB-FIFO.
    ("usb_fifo_clock", 0, Pins("D17"), IOStandard("LVCMOS33")),
    ("usb_fifo", 0,
        Subsignal("rst", Pins("K22")),
        Subsignal("data", Pins(
            "A16 F14 A15 F13 A14 E14 A13 E13",
            "B13 C15 C13 C14 B16 E17 B15 F16",
            "A20 E18 B20 F18 D19 D21 E19 E21",
            "A21 B21 A19 A18 F20 F19 B18 B17")),
        Subsignal("be",    Pins("K16 L16 G20 H20")),
        Subsignal("rxf_n", Pins("M13")),
        Subsignal("txe_n", Pins("L13")),
        Subsignal("rd_n",  Pins("K19")),
        Subsignal("wr_n",  Pins("M15")),
        Subsignal("oe_n",  Pins("K18")),
        Subsignal("siwua", Pins("M16")),
        IOStandard("LVCMOS33"),
        Misc("SLEW=FAST")
    ),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk100"
    default_clk_period = 1e9/100e6

    def __init__(self):
        Xilinx7SeriesPlatform.__init__(self, "xc7a35t-fgg484-2", _io, toolchain="vivado")
        self.toolchain.bitstream_commands = \
            ["set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]",
             "set_property BITSTREAM.CONFIG.CONFIGRATE 40 [current_design]"]
        self.toolchain.additional_commands = \
            ["write_cfgmem -force -format bin -interface spix4 -size 16 "
             "-loadbit \"up 0x0 {build_name}.bit\" -file {build_name}.bin"]

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk100", loose=True), 1e9/100e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2019 Tom Keddie <git@bronwenandtom.com>
# Copyright (c) 2019 Sean Cross <sean@xobs.io>
# SPDX-License-Identifier: BSD-2-Clause

# Fomu EVT board:
# - Crowd Supply campaign: https://www.crowdsupply.com/sutajio-kosagi/fomu
# - Design files: https://github.com/im-tomu/fomu-hardware/tree/evt/hardware/pcb

from litex.build.generic_platform import *
from litex.build.lattice import LatticeiCE40Platform
from litex.build.lattice.programmer import IceStormProgrammer

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk48", 0, Pins("44"), IOStandard("LVCMOS33")),

    # Leds
    ("user_led_n", 0, Pins("41"), IOStandard("LVCMOS33")),
    ("rgb_led", 0,
        Subsignal("r", Pins("40")),
        Subsignal("g", Pins("39")),
        Subsignal("b", Pins("41")),
        IOStandard("LVCMOS33"),
    ),

    # Buttons
    ("user_btn_n", 0, Pins("42"), IOStandard("LVCMOS33")),
    ("user_btn_n", 1, Pins("38"), IOStandard("LVCMOS33")),

    # Serial
    ("serial", 0,
        Subsignal("rx", Pins("21")),
        Subsignal("tx", Pins("13"), Misc("PULLUP")),
        IOStandard("LVCMOS33")
    ),

    # USB
    ("usb", 0,
        Subsignal("d_p", Pins("34")),
        Subsignal("d_n", Pins("37")),
        Subsignal("pullup",   Pins("35")),
        Subsignal("pulldown", Pins("36")),
        IOStandard("LVCMOS33")
    ),

    # SPIFlash
    ("spiflash", 0,
        Subsignal("cs_n", Pins("16"), IOStandard("LVCMOS33")),
        Subsignal("clk",  Pins("15"), IOStandard("LVCMOS33")),
        Subsignal("miso", Pins("17"), IOStandard("LVCMOS33")),
        Subsignal("mosi", Pins("14"), IOStandard("LVCMOS33")),
        Subsignal("wp",   Pins("18"), IOStandard("LVCMOS33")),
        Subsignal("hold", Pins("19"), IOStandard("LVCMOS33")),
    ),

    ("spiflash4x", 0,
        Subsignal("cs_n", Pins("16"), IOStandard("LVCMOS33")),
        Subsignal("clk",  Pins("15"), IOStandard("LVCMOS33")),
        Subsignal("dq",   Pins("14 17 18 19"), IOStandard("LVCMOS33")),
    ),

    # I2C
    ("i2c", 0,
        Subsignal("scl", Pins("12"), IOStandard("LVCMOS18")),
        Subsignal("sda", Pins("20"), IOStandard("LVCMOS18")),
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    ("touch_pins", "48 47 46 45"),
    ("pmoda_n",    "28 27 26 23"),
    ("pmodb_n",    "48 47 46 45"),
    ("dbg",        "20 12 11 25 10 9"),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(LatticeiCE40Platform):
    default_clk_name   = "clk48"
    default_clk_period = 1e9/48e6

    def __init__(self, toolchain="icestorm"):
        LatticeiCE40Platform.__init__(self, "ice40-up5k-sg48", _io, _connectors, toolchain=toolchain)

    def create_programmer(self):
        return IceStormProgrammer()

    def do_finalize(self, fragment):
        LatticeiCE40Platform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk48", loose=True), 1e9/48e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Florent Kermarrec <florent@enjoy-digital.fr>
# Copyright (c) 2022 Lukas F. Hartmann, MNT Research GmbH <lukas@mntre.com>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform
from litex.build.openocd import OpenOCD

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst.
    ("clk100", 0, Pins("AA10"), IOStandard("LVCMOS15")),

    # Serial.
    ("serial", 0,
        Subsignal("tx", Pins("D15")),
        Subsignal("rx", Pins("C18")),
        IOStandard("LVCMOS33")
    ),
    ("serial", 1,
        Subsignal("tx", Pins("H16")),
        Subsignal("rx", Pins("G16")),
        IOStandard("LVCMOS33")
    ),
    ("debug_serial", 0,
        Subsignal("tx", Pins("C17")),
        Subsignal("rx", Pins("C16")),
        IOStandard("LVCMOS33")
    ),

    # SPIFlash
    ("spiflash4x", 0,  # clock needs to be accessed through STARTUPE2
        Subsignal("cs_n", Pins("C23")),
        Subsignal("dq",   Pins("B24 A25 B22 A22")),
        IOStandard("LVCMOS33")
    ),

    # SDCard.
    ("spisdcard", 0,
        Subsignal("clk",  Pins("C11")),
        Subsignal("mosi", Pins("A15"), Misc("PULLUP True")),
        Subsignal("cs_n", Pins("B15"), Misc("PULLUP True")),
        Subsignal("miso", Pins("A14"), Misc("PULLUP True")),
        Misc("SLEW=FAST"),
        IOStandard("LVCMOS18"),
    ),
    ("sdcard", 0,
        Subsignal("data", Pins("A14 B10 A12 B15"), Misc("PULLUP True")),
        Subsignal("cmd",  Pins("A15"), Misc("PULLUP True")),
        Subsignal("clk",  Pins("C11")),
        Subsignal("cd",   Pins("A10")),
        Misc("SLEW=FAST"),
        IOStandard("LVCMOS18"),
    ),

    # RGMII Ethernet
    ("eth_refclk", 0, Pins("F17"), IOStandard("LVCMOS33")), # CHECKME: Drive it?
    ("eth_clocks", 0,
        Subsignal("tx", Pins("E18")),
        Subsignal("rx", Pins("D18")),
        IOStandard("LVCMOS33")
    ),
    ("eth", 0,
        Subsignal("rst_n",   Pins("G17"), IOStandard("LVCMOS33")),
        Subsignal("int_n",   Pins("E16"), IOStandard("LVCMOS33")),
        Subsignal("mdio",    Pins("E15"), IOStandard("LVCMOS33")),
        Subsignal("mdc",     Pins("E17"), IOStandard("LVCMOS33")),
        Subsignal("rx_ctl",  Pins("F15"), IOStandard("LVCMOS33")),
        Subsignal("rx_data", Pins("J15 J16 F20 D20"), IOStandard("LVCMOS33")),
        Subsignal("tx_ctl",  Pins("D19"), IOStandard("LVCMOS33")),
        Subsignal("tx_data", Pins("H18 H17 G19 F18"), IOStandard("LVCMOS33")),
    ),

    # I2C
    ("i2c", 0,
        Subsignal("scl", Pins("Y26")),
        Subsignal("sda", Pins("W26")),
        IOStandard("LVCMOS18"),
    ),
    ("i2c", 1,
        Subsignal("scl", Pins("G12")),
        Subsignal("sda", Pins("A13")),
        IOStandard("LVCMOS18"),
    ),
    ("i2c", 2,
        Subsignal("scl", Pins("H26")),
        Subsignal("sda", Pins("G26")),
        IOStandard("LVCMOS33"),
    ),

    # GPIO
    ("resets", 0,
        Pins("M21 M22 C13 C14"), # Backlight PWM, Backlight EN, hdmi_rst_n, edp_reset_n
        IOStandard("LVCMOS18")
    ),
    ("gpio", 0,
        Pins("D13 N19 M19"), # USB hub reset, Analogix reset, eDP HPD
        IOStandard("LVCMOS18")
    ),

    # DDR3 SDRAM.
    ("ddram", 0,
        Subsignal("a",       Pins(
            "AC8 AA9 AA7 AD9 Y8 AA8 W11 Y10",
            "Y11 Y7 AC11 V11 AB11 V7 V9 V8"),
            IOStandard("SSTL15")),
        Subsignal("ba",      Pins("AC7 AB7 AB9"), IOStandard("SSTL15")),
        Subsignal("ras_n",   Pins("AF7"), IOStandard("SSTL15")),
        Subsignal("cas_n",   Pins("AE7"), IOStandard("SSTL15")),
        Subsignal("we_n",    Pins("AC9"), IOStandard("SSTL15")),
        Subsignal("cs_n",    Pins("AD8"), IOStandard("SSTL15")),
        Subsignal("dm",      Pins(
            "U6 Y3 AB6 AD4"),
            IOStandard("SSTL15")),
        Subsignal("dq",      Pins(
            " V4  W3  U5  U2  U7  U1  V6  V3",
            " Y2  Y1 AA3  V2 AC2  W1 AB2  V1",
            "AA4 AB4 AC4 AC3 AC6 Y6 Y5 AD6",
            "AD1 AE1 AE3 AE2 AE6 AE5 AF3 AF2"
            ),
            IOStandard("SSTL15_T_DCI")),
        Subsignal("dqs_p",   Pins("W6 AB1 AA5 AF5"),
            IOStandard("DIFF_SSTL15")),
        Subsignal("dqs_n",   Pins("W5 AC1 AB5 AF4"),
            IOStandard("DIFF_SSTL15")),
        Subsignal("clk_p",   Pins("W10"),  IOStandard("DIFF_SSTL15")),
        Subsignal("clk_n",   Pins("W9"),   IOStandard("DIFF_SSTL15")),
        Subsignal("cke",     Pins("AB12"), IOStandard("SSTL15")),
        Subsignal("odt",     Pins("AC12"), IOStandard("SSTL15")),
        Subsignal("reset_n", Pins("AA2"),  IOStandard("LVCMOS15")),
        Misc("SLEW=FAST"),
        Misc("VCCAUX_IO=HIGH")
    ),

    # HDMI (DISP1)
    ("hdmi", 0,
        Subsignal("clk",     Pins("C12")),
        Subsignal("de",      Pins("D10")),
        Subsignal("hsync_n", Pins("D11")),
        Subsignal("vsync_n", Pins("E11")),
        Subsignal("b", Pins("H13 G10 J13 H12 J10  H8  H9 J11")), # [16:23]
        Subsignal("g", Pins("F14  C9 G14 F10 H14 G11 H11  G9")), # [8:15]
        Subsignal("r", Pins("E10  D8  F9  F8  A9  A8  B9  D9")), # [0:7]
        IOStandard("LVCMOS18")
    ),

    # RGB->eDP (DISP2)
    ("edp", 0,
        Subsignal("clk",     Pins("AC18")),
        Subsignal("de",      Pins("AA15")),
        Subsignal("hsync", Pins("AB15")), # hsync_n for negative
        Subsignal("vsync", Pins("AB16")), # vsync_n for negative
        Subsignal("r", Pins("AF14 AF15 AE15 AE16 AF17 AE17 AA14 AF18")), # [16:23]
        Subsignal("g", Pins("AD15 AE18 AD16 AF19 AC16 AD14 AC17 AC14")), # [8:15]
        Subsignal("b", Pins("AB14 Y15  AA17 AA18 Y16  AF20 AD20 AB17")), # [0:7]
        IOStandard("LVCMOS18"), Misc("DRIVE=4"),
    ),

    ("edpoff", 0,
       Pins("AD18"),
       IOStandard("LVCMOS18")
    ),

    # Backlight via Motherboard (unused)
    ("backlight", 0,
        Subsignal("pwm", Pins("K16")),
        Subsignal("en", Pins("B16")),
        IOStandard("LVCMOS33")
    ),

    # USB
    ("usb", 0,
        Subsignal("dp", Pins("D26")),
        Subsignal("dm", Pins("C26")),
        IOStandard("LVCMOS33"),
    ),
    ("usb_pull", 0,
        Pins("F25"),
        IOStandard("LVCMOS33")
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = []

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk100"
    default_clk_period = 1e9/100e6

    def __init__(self, toolchain="vivado"):
        Xilinx7SeriesPlatform.__init__(self, "xc7k325t-ffg676-2", _io, _connectors, toolchain=toolchain)
        # Enable bitstream compression, quad SPI and 50MHz rate for quick boot from SPI flash
        # see https://github.com/timvideos/litex-buildenv/issues/79
        self.toolchain.bitstream_commands = [
            "set_property BITSTREAM.GENERAL.COMPRESS True [current_design]",
            "set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]",
            "set_property BITSTREAM.CONFIG.CONFIGRATE 50 [current_design]"
        ]

    def create_programmer(self):
        return OpenOCD("openocd_xc7_ft2232.cfg", "bscan_spi_xc7a325t.bit")

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk100", loose=True), 1e9/100e6)
        self.add_platform_command("set_property INTERNAL_VREF 0.750 [get_iobanks 33]")
        self.add_platform_command("set_property INTERNAL_VREF 0.750 [get_iobanks 34]")



from litex.build.generic_platform import *
from litex.build.lattice import LatticeiCE40Platform
from litex.build.lattice.programmer import IceStormProgrammer


_io = [
    # Clk
    ("clk12", 0, Pins("35"), IOStandard("LVCMOS33")),

    # LEDs
    ("user_led_n", 0, Pins("39"), IOStandard("LVCMOS33")),
    ("rgb_led", 0,
        Subsignal("r", Pins("41")),
        Subsignal("g", Pins("40")),
        Subsignal("b", Pins("39")),
        IOStandard("LVCMOS33")
    ),
    
    # Buttons
    ("user_sw", 0, Pins("23"), IOStandard("LVCMOS33")),
    ("user_sw", 1, Pins("25"), IOStandard("LVCMOS33")),
    ("user_sw", 2, Pins("34"), IOStandard("LVCMOS33")),
    ("user_sw", 3, Pins("43"), IOStandard("LVCMOS33"))
]

spiflash = [
    # Only usable in PROG FLASH mode and J7 attached (see PCB silkscreen).
    ("spiflash", 0,
        Subsignal("cs_n", Pins("16"), IOStandard("LVCMOS33")),
        Subsignal("clk", Pins("15"), IOStandard("LVCMOS33")),
        Subsignal("mosi", Pins("14"), IOStandard("LVCMOS33")),
        Subsignal("miso", Pins("17"), IOStandard("LVCMOS33")),
    ),
]

serial = [
    ("serial", 0,
        Subsignal("tx", Pins("J3:0")),
        Subsignal("rx", Pins("J3:1")),
        IOStandard("LVCMOS33")
    ),
    ("serial", 1,
        Subsignal("tx", Pins("J3:2")),
        Subsignal("rx", Pins("J3:3")),
        IOStandard("LVCMOS33")
    )
]


_connectors = [
    # Many pins on the AARDVARK, PMOD, J52/HEADER A, and J2/HEADER B connectors
    # are multiplexed with other I/O or connector pins. For completeness, all
    # pins are exposed here except Vdd, NC, and GND. Pin order is as specified
    # on the schematic (except for PMOD, which uses Digilent's numbering).

    # AARDVARK connector omitted- although sysCONFIG pins are exposed on this
    # header (which can be used as GPIO), it is meant for flashing using an
    # external programmer rather than as an I/O port.

    # PMOD connector shares pins with sysCONFIG- make sure to remove jumper
    # J7 if using the PMOD. TODO: Perhaps it would be better to split into two
    # single 6-pin PMODs.
    #
    # PMOD pinout (using ICE40 pin names):
    # 1, 2, 3, 4- SPI_SS, SPI_SI, SPI_SO, SPI_SCK
    # 5, 6, 7, 8- Free
    ("PMOD", "16   17   14  15  27  26  32  31"),
    #Silk:    SS MOSI MISO SCK 38B 39A 43A 42B

    # J52 exposes LEDs and sysCONFIG pins (yet again). Make sure to remove
    # jumper J7 if using the PMOD. Pin order is as follows (right to left):
    # 12 10 8 6 4 2
    # 11  9 7 5 3 1
    #
    # J52's pinout (using ICE40 pin names for SPI flash):
    # 1, 2- Vdd
    # 3, 4- rgb_led.b, SPI_SI
    # 5, 6- rgb_led.g, SPI_SO
    # 7, 8- GND, SPI_SCK
    # 9, 10- rgb_led.r, SPI_SS
    # 11, 12- GND
    #         3  4  5  6  8  9 10
    ("J52", "39 17 40 14 15 41 16"),

    # Pin order of J2, and J3 are as follows (left to right/top to bottom):
    # 2 4 6 8 10 12 14 16 18 20
    # 1 3 5 7 9  11 13 15 17 19
    #
    # J2's pinout is shared by many things. Entire pinout listed follows:
    # 1, 2- Vdd
    # 3, 4- user_sw0, NC
    # 5, 6- user_sw1, NC
    # 7, 8- PMOD D5, Free
    # 9, 10- PMOD D4, Free
    # 11, 12- PMOD D6, Free
    # 13, 14- PMOD D7, Free
    # 15, 16- Free, 12.00 clock
    # 17, 18- user_sw2, GND
    # 19, 20- user_sw3, GND
    #        3   5   7   8   9  10  11  12  13  14     15      16  17  19
    ("J2", "23  25  26  36  27  42  32  38  31  28     37      35  34  43"),
    # Silk:37A 36B 39A 48B 38B 51A 43A 50B 42B 41A 45A_G1 ICE_CLK 44B 49A 
    # index: 0   1   2   3   4  5    6   7   8   9     10      11  12  13

    # Bank2: 4 3 48 45 47 44 46 2
    # Bank1: 12 21 13 20 19 18 11 10 9 6
    # J3's pinout is all Free, except 1 (Vdd) and 19 (GND).
    #        2  3   4  5   6  7      8  9  10 11  12    13  14 15  16 17  18  20
    ("J3", "12  4  21  3  13 48     20 45  19 47  18    44  11 46  10  2   9   6"),
    # Silk:22A 8A 23B 9B 24A 4A 25B_G3 5B 29B 2A 31B 3B_G6 20A 0A 18A 6A 16A 13B
    # index: 0  1   2  3   4  5      6  7   8  9  10    11  12 13  14 15  16  17
]


class Platform(LatticeiCE40Platform):
    default_clk_name = "clk12"
    default_clk_period = 1e9/12e6

    def __init__(self, toolchain="icestorm"):
        LatticeiCE40Platform.__init__(self, "ice40-up5k-sg48", _io, _connectors, toolchain=toolchain)
        self.add_extension(serial)
        self.add_extension(spiflash)

    def create_programmer(self):
        return IceStormProgrammer()

    def do_finalize(self, fragment):
        LatticeiCE40Platform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk12", loose=True), 1e9/12e6)




#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2019 David Shah <dave@ds0.me>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.lattice import LatticeECP5Platform
from litex.build.lattice.programmer import OpenOCDJTAGProgrammer

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk / Rst
    ("clk100", 0, Pins("B29"), IOStandard("LVDS")), # [broken on rev1.0 (non diff pair)]
    ("clk12",  0, Pins("B3"),  IOStandard("LVCMOS33")),
    ("clkref", 0, Pins("E17"), IOStandard("LVCMOS33")),

    # Leds
    ("user_led",  0, Pins("C26"),  IOStandard("LVCMOS33"), Misc("PULLMODE=NONE")),
    ("user_led",  1, Pins("D26"),  IOStandard("LVCMOS33"), Misc("PULLMODE=NONE")),
    ("user_led",  2, Pins("A28"),  IOStandard("LVCMOS33"), Misc("PULLMODE=NONE")),
    ("user_led",  3, Pins("A29"),  IOStandard("LVCMOS33"), Misc("PULLMODE=NONE")),
    ("user_led",  4, Pins("A30"),  IOStandard("LVCMOS33"), Misc("PULLMODE=NONE")),
    ("user_led",  5, Pins("AK29"), IOStandard("LVCMOS33"), Misc("PULLMODE=NONE")),
    ("user_led",  6, Pins("AH32"), IOStandard("LVCMOS33"), Misc("PULLMODE=NONE")),
    ("user_led",  7, Pins("AH30"), IOStandard("LVCMOS33"), Misc("PULLMODE=NONE")),
    ("user_led",  8, Pins("AH28"), IOStandard("LVCMOS33"), Misc("PULLMODE=NONE")),
    ("user_led",  9, Pins("AG30"), IOStandard("LVCMOS33"), Misc("PULLMODE=NONE")),
    ("user_led", 10, Pins("AG29"), IOStandard("LVCMOS33"), Misc("PULLMODE=NONE")),
    ("user_led", 11, Pins("AK30"), IOStandard("LVCMOS33"), Misc("PULLMODE=NONE")),

    # Buttons
    ("user_btn", 0, Pins("Y32"),  IOStandard("SSTL135_I")),
    ("user_btn", 1, Pins("W31"),  IOStandard("SSTL135_I")),
    ("user_btn", 2, Pins("AD30"), IOStandard("SSTL135_I")),
    ("user_btn", 3, Pins("AD29"), IOStandard("SSTL135_I")),

    # Switches
    ("user_dip", 0, Pins("AE31"), IOStandard("SSTL135_I")),
    ("user_dip", 1, Pins("AE32"), IOStandard("SSTL135_I")),
    ("user_dip", 2, Pins("AD32"), IOStandard("SSTL135_I")),
    ("user_dip", 3, Pins("AC32"), IOStandard("SSTL135_I")),
    ("user_dip", 4, Pins("AB32"), IOStandard("SSTL135_I")),
    ("user_dip", 5, Pins("AB31"), IOStandard("SSTL135_I")),
    ("user_dip", 6, Pins("AC31"), IOStandard("SSTL135_I")),
    ("user_dip", 7, Pins("AC30"), IOStandard("SSTL135_I")),

    # Serial
    ("serial", 0,
        Subsignal("rx", Pins("AM28"), IOStandard("LVCMOS33")),
        Subsignal("tx", Pins("AL28"), IOStandard("LVCMOS33")),
    ),

    # USB FIFO
    ("usb_fifo", 0,
        Subsignal("dq",    Pins("AM28 AL28 AM29 AK28 AK32 AM30 AJ32 AL30"), IOStandard("LVCMOS33")),
        Subsignal("txe_n", Pins("AM31"),  IOStandard("LVCMOS33")),
        Subsignal("rxf_n", Pins("AJ31"),  IOStandard("LVCMOS33")),
        Subsignal("rd_n",  Pins("AL32"),  IOStandard("LVCMOS33")),
        Subsignal("wr_n",  Pins("AG28"),  IOStandard("LVCMOS33")),
        Subsignal("siwu_n", Pins("AJ28"), IOStandard("LVCMOS33")),
    ),

    # DDR3 SDRAM
    ("dram_vtt_en", 0, Pins("E25"), IOStandard("LVCMOS33")),
    ("ddram", 0,
        Subsignal("a", Pins(
            "E30 F28 C32 E29 F32 D30 E32 D29",
            "D32 C31 H32 F31 F29 B32 D31"),
            IOStandard("SSTL135_I")),
        Subsignal("ba",    Pins("H31 H30 J30"), IOStandard("SSTL135_I")),
        Subsignal("ras_n", Pins("K31"), IOStandard("SSTL135_I")),
        Subsignal("cas_n", Pins("K30"), IOStandard("SSTL135_I")),
        Subsignal("we_n",  Pins("J32"), IOStandard("SSTL135_I")),
        Subsignal("cs_n",  Pins("K29"), IOStandard("SSTL135_I")),
        Subsignal("dm", Pins("R26 L27 Y27 U31"), IOStandard("SSTL135_I")),
        Subsignal("dq", Pins(
            " V26  R27  V27  T26  U28  T27  T29  U26",
            " P27  K28  P26  L26  K27  N26  L29  K26",
            "AC27  W28 AC26  Y26 AB26  W29 AD26  Y28",
            " T32  U32  P31  V32  P32  W32  N32  U30"),
            IOStandard("SSTL135_I"),
            Misc("TERMINATION=75")),
        Subsignal("dqs_p", Pins("R29 N30 AB28 R32"), IOStandard("SSTL135D_I"),
            Misc("TERMINATION=OFF"),
            Misc("DIFFRESISTOR=100")),
        Subsignal("clk_p", Pins("L31"), IOStandard("SSTL135D_I")),
        Subsignal("cke",   Pins("K32"), IOStandard("SSTL135_I")),
        Subsignal("odt",   Pins("J29"), IOStandard("SSTL135_I")),
        Subsignal("reset_n", Pins("L32"), IOStandard("SSTL135_I")),
        Misc("SLEWRATE=FAST"),
    ),

    # RGMII Ethernet
    ("eth_clocks", 0,
        Subsignal("tx",  Pins("A15")),
        Subsignal("rx",  Pins("C17")),
        Subsignal("ref", Pins("A17")),
        IOStandard("LVCMOS33")
    ),
    ("eth", 0,
        Subsignal("rst_n",   Pins("D16")),
        Subsignal("int_n",   Pins("E16")),
        Subsignal("mdio",    Pins("F17")),
        Subsignal("mdc",     Pins("B17")),
        Subsignal("rx_ctl",  Pins("A16")),
        Subsignal("rx_data", Pins("C16 B16 B14 F16")),
        Subsignal("tx_ctl",  Pins("D15")),
        Subsignal("tx_data", Pins("A14 F15 C15 C14")),
        IOStandard("LVCMOS33")
    ),

    # I2C Clock Generator
    ("clkgen", 0,
        Subsignal("sda",   Pins("C22")),
        Subsignal("scl",   Pins("A22")),
        Subsignal("sd_oe", Pins("A2")),
        IOStandard("LVCMOS33")
    ),

    # PCIe
    ("pcie_x2", 0,
        Subsignal("clk_p",  Pins("AM14")),
        Subsignal("clk_n",  Pins("AM15")),
        Subsignal("rx_p",   Pins("AM8  AK12")),
        Subsignal("rx_n",   Pins("AM9  AK13")),
        Subsignal("tx_p",   Pins("AK9  AM11")),
        Subsignal("tx_n",   Pins("AK10 AM12")),
        Subsignal("perst",  Pins("D22"), IOStandard("LVCMOS33")),
        Subsignal("wake_n", Pins("A23"), IOStandard("LVCMOS33")),
    ),

    # M2
    ("m2", 0,
        Subsignal("clk_p", Pins("AM23")),
        Subsignal("clk_n", Pins("AM24")),
        Subsignal("rx_p",  Pins("AM17 AK21")),
        Subsignal("rx_n",  Pins("AM18 AK22")),
        Subsignal("tx_p",  Pins("AK18 AM20")),
        Subsignal("tx_n",  Pins("AK19 AM21")),

        Subsignal("clksel", Pins("N3"), IOStandard("LVCMOS33")),

        Subsignal("sdio_clk", Pins("L4"), IOStandard("LVCMOS33")),
        Subsignal("sdio_cmd", Pins("K4"), IOStandard("LVCMOS33")),
        Subsignal("sdio_dq",  Pins("L7 N4 L6 N6"), IOStandard("LVCMOS33")),

        Subsignal("uart_tx",    Pins("P6"), IOStandard("LVCMOS33")),
        Subsignal("uart_rx",    Pins("K5"), IOStandard("LVCMOS33")),
        Subsignal("uart_rts_n", Pins("N7"), IOStandard("LVCMOS33")),
        Subsignal("uart_cts_n", Pins("P7"), IOStandard("LVCMOS33"))
    ),

    # SDCard
    ("spisdcard", 0,
        Subsignal("clk",  Pins("AK3")),
        Subsignal("mosi", Pins("AH3"), Misc("PULLMODE=UP")),
        Subsignal("cs_n", Pins("AK1"), Misc("PULLMODE=UP")),
        Subsignal("miso", Pins("AG1"), Misc("PULLMODE=UP")),
        Misc("SLEWRATE=FAST"),
        IOStandard("LVCMOS33"),
    ),
    ("sdcard", 0,
        Subsignal("clk",  Pins("AK3")),
        Subsignal("cmd",  Pins("AH3"), Misc("PULLMODE=UP")),
        Subsignal("data", Pins("AG1 AJ1 AH1 AK1"), Misc("PULLMODE=UP")),
        Misc("SLEWRATE=FAST"),
        IOStandard("LVCMOS33"),
    ),


    # SPIFlash
    ("spiflash", 0,
        Subsignal("clk",  Pins("AM3")),
        Subsignal("cs_n", Pins("AJ3")),
        Subsignal("mosi", Pins("AK2")),
        Subsignal("miso", Pins("AJ2")),
        Subsignal("wp",   Pins("AM2")),
        Subsignal("hold", Pins("AL1")),
        IOStandard("LVCMOS33")
    ),
    ("spiflash4x", 0,
        Subsignal("clk",  Pins("AM3")),
        Subsignal("cs_n", Pins("AJ3")),
        Subsignal("dq",   Pins("AK2 AJ2 AM2 AL1")),
        IOStandard("LVCMOS33")
    ),

    # USB ULPI
    ("ulpi", 0,
        Subsignal("clk",   Pins("A18")),
        Subsignal("stp",   Pins("D18")),
        Subsignal("dir",   Pins("C18")),
        Subsignal("nxt",   Pins("F18")),
        Subsignal("reset", Pins("D17")),
        Subsignal("data",  Pins("C20 C19 E19 D20 A20 B19 D19 A19")),
        IOStandard("LVCMOS33")
    ),

    # HDMI
    ("hdmi", 0,
        Subsignal("r", Pins("F10  F9  D9  D8  C7  F8  E8 D11")),
        Subsignal("g", Pins("B8   A7  C8  C9 F11 E11 E10 D10")),
        Subsignal("b", Pins("C11 A11 B11 A10 B10 C10  A8  B7")),
        Subsignal("de",      Pins("F14")),
        Subsignal("clk",     Pins("A9")),
        Subsignal("vsync_n", Pins("E14")),
        Subsignal("hsync_n", Pins("F13")),
        Subsignal("sda", Pins("D13")),
        Subsignal("scl", Pins("C13")),
        IOStandard("LVCMOS33")
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    ("pmoda", "F19 F20 B22 C23 D14 A13 E22 D23"),
    ("pmodb", "C25 A26 F23 F25 B25 D25 F22 F24"),
    ("pmodx", "A24 C24 D24 B23 D23 A25"),

    ("ext0", "T1 U1 AE5 AE4 AB5 AB6 Y5 W5 W2 Y1 AB7 AC6 AB3 AB4 AD3 AE3 AB1 AC1 AD1 AE1 AD6 AE6 AC7 AD7"),
    ("ext1", "P5 P4 R7 T7 R6 T6 U6 U7 R4 T5 T4 U5 U4 V4 V6 V7 P2 P3 R3 T3 N1 P1 U2 U3"),
    ("ext2", "K6 K7 J7 J6 H6 H5 F4 F5 F3 E3 C4 C3 C5 D5 D3 D2 H2 H3 J3 K3 B1 C2 F1 H1")
]


# PMODS --------------------------------------------------------------------------------------------

def raw_pmod_io(pmod):
    return [(pmod, 0, Pins(" ".join([f"{pmod}:{i:d}" for i in range(8)])), IOStandard("LVCMOS33"))]

def sdcard_pmod_io(pmod):
    return [
        # SDCard PMOD:
        # - https://store.digilentinc.com/pmod-microsd-microsd-card-slot/
        # - https://github.com/antmicro/arty-expansion-board
        ("spisdcard", 0,
            Subsignal("clk",  Pins(f"{pmod}:3")),
            Subsignal("mosi", Pins(f"{pmod}:1"), Misc("PULLMODE=UP")),
            Subsignal("cs_n", Pins(f"{pmod}:0"), Misc("PULLMODE=UP")),
            Subsignal("miso", Pins(f"{pmod}:2"), Misc("PULLMODE=UP")),
            Misc("SLEWRATE=FAST"),
            IOStandard("LVCMOS33"),
        ),
        ("sdcard", 0,
            Subsignal("data", Pins(f"{pmod}:2 {pmod}:4 {pmod}:5 {pmod}:0"), Misc("PULLMODE=UP")),
            Subsignal("cmd",  Pins(f"{pmod}:1"), Misc("PULLMODE=UP")),
            Subsignal("clk",  Pins(f"{pmod}:3")),
            Subsignal("cd",   Pins(f"{pmod}:6")),
            Misc("SLEWRATE=FAST"),
            IOStandard("LVCMOS33"),
        ),
]
_sdcard_pmod_io = sdcard_pmod_io("pmoda") # SDCARD PMOD on PMODA.

# Platform -----------------------------------------------------------------------------------------

class Platform(LatticeECP5Platform):
    default_clk_name   = "clk12"
    default_clk_period = 1e9/12e6

    def __init__(self, toolchain="trellis", **kwargs):
        LatticeECP5Platform.__init__(self, "LFE5UM5G-85F-8BG756C", _io, _connectors, toolchain=toolchain, **kwargs)

    def create_programmer(self):
        return OpenOCDJTAGProgrammer("openocd_trellisboard.cfg")

    def do_finalize(self, fragment):
        LatticeECP5Platform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk100",        loose=True), 1e9/100e6)
        self.add_period_constraint(self.lookup_request("clk12",         loose=True), 1e9/12e6)
        self.add_period_constraint(self.lookup_request("eth_clocks:rx", loose=True), 1e9/125e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2019-2020 Florent Kermarrec <florent@enjoy-digital.fr>
# Copyright (c) 2022 Oliver Szabo <16oliver16@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.xilinx import Xilinx7SeriesPlatform, VivadoProgrammer

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Leds
    ("user_led", 0, Pins("M14"), IOStandard("LVCMOS33")),
    ("user_led", 1, Pins("M15"), IOStandard("LVCMOS33")),
    ("user_led", 2, Pins("G14"), IOStandard("LVCMOS33")),
    ("user_led", 3, Pins("D18"), IOStandard("LVCMOS33")),

    # Switches
    ("user_sw", 0, Pins("G15"), IOStandard("LVCMOS33")),
    ("user_sw", 1, Pins("P15"), IOStandard("LVCMOS33")),
    ("user_sw", 2, Pins("W13"), IOStandard("LVCMOS33")),
    ("user_sw", 3, Pins("T16"), IOStandard("LVCMOS33")),

    # HDMI Out
    ("hdmi_out", 0,
        Subsignal("clk_p",   Pins("H16"), IOStandard("TMDS_33")),
        Subsignal("clk_n",   Pins("H17"), IOStandard("TMDS_33")),
        Subsignal("data0_p", Pins("D19"), IOStandard("TMDS_33")),
        Subsignal("data0_n", Pins("D20"), IOStandard("TMDS_33")),
        Subsignal("data1_p", Pins("C20"), IOStandard("TMDS_33")),
        Subsignal("data1_n", Pins("B20"), IOStandard("TMDS_33")),
        Subsignal("data2_p", Pins("B19"), IOStandard("TMDS_33")),
        Subsignal("data2_n", Pins("A20"), IOStandard("TMDS_33")),
        Subsignal("scl",     Pins("G17"), IOStandard("LVCMOS33")),
        Subsignal("sda",     Pins("G18"), IOStandard("LVCMOS33")),
        Subsignal("cec",     Pins("E19"), IOStandard("LVCMOS33")),
        Subsignal("hdp",     Pins("E18"), IOStandard("LVCMOS33")),
    ),

    # Serial
    ("serial", 0,
        Subsignal("tx", Pins("T17")),
        Subsignal("rx", Pins("Y17")),
        IOStandard("LVCMOS33")
    ),
]

_io_z7 = [
    # Clk / Rst
    ("clk125", 0, Pins("K17"), IOStandard("LVCMOS33")),

    # Buttons
    ("user_btn", 0, Pins("K18"), IOStandard("LVCMOS33")),
    ("user_btn", 1, Pins("P16"), IOStandard("LVCMOS33")),
    ("user_btn", 2, Pins("K19"), IOStandard("LVCMOS33")),
    ("user_btn", 3, Pins("Y16"), IOStandard("LVCMOS33")),
]

_io_original = [
    # Clk / Rst
    ("clk125", 0, Pins("L16"), IOStandard("LVCMOS33")),

    # Buttons
    ("user_btn", 0, Pins("R18"), IOStandard("LVCMOS33")),
    ("user_btn", 1, Pins("P16"), IOStandard("LVCMOS33")),
    ("user_btn", 2, Pins("V16"), IOStandard("LVCMOS33")),
    ("user_btn", 3, Pins("Y16"), IOStandard("LVCMOS33")),
]

_ps7_io = [
    # PS7
    ("ps7_clk",   0, Pins(1)),
    ("ps7_porb",  0, Pins(1)),
    ("ps7_srstb", 0, Pins(1)),
    ("ps7_mio",   0, Pins(54)),
    ("ps7_ddram", 0,
        Subsignal("addr",    Pins(15)),
        Subsignal("ba",      Pins(3)),
        Subsignal("cas_n",   Pins(1)),
        Subsignal("ck_n",    Pins(1)),
        Subsignal("ck_p",    Pins(1)),
        Subsignal("cke",     Pins(1)),
        Subsignal("cs_n",    Pins(1)),
        Subsignal("dm",      Pins(4)),
        Subsignal("dq",      Pins(32)),
        Subsignal("dqs_n",   Pins(4)),
        Subsignal("dqs_p",   Pins(4)),
        Subsignal("odt",     Pins(1)),
        Subsignal("ras_n",   Pins(1)),
        Subsignal("reset_n", Pins(1)),
        Subsignal("we_n",    Pins(1)),
        Subsignal("vrn",     Pins(1)),
        Subsignal("vrp",     Pins(1)),
    ),
]

_usb_uart_pmod_io = [
    # USB-UART PMOD on JB:
    # - https://store.digilentinc.com/pmod-usbuart-usb-to-uart-interface/
    ("usb_uart", 0,
        Subsignal("tx", Pins("pmodb:1")),
        Subsignal("rx", Pins("pmodb:2")),
        IOStandard("LVCMOS33")
    ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    ("pmoda", "N15 L14 K16 K14 N16 L15 J16 J14"), # XADC
    ("pmodc", "V15 W15 T11 T10 W14 Y14 T12 U12"),
    ("pmodd", "T14 T15 P14 R14 U14 U15 V17 V18"),
    ("pmode", "V12 W16 J15 H15 V13 U17 T17 Y17"),
]

_connectors_z7 = [
    ("pmodb", "V8  W8  U7  V7  Y7  Y6  V6  W6")
]

_connectors_original = [
    ("pmodb", "T20  U20  V20  W20  Y18  Y19  W18  W19")
]

ps7_config_variants = {
    "common" : {
        "PCW_FPGA_FCLK0_ENABLE"         : "1",
        "PCW_UART1_BAUD_RATE"           : "115200",
        "PCW_EN_UART1"                  : "1",
        "PCW_UART1_PERIPHERAL_ENABLE"   : "1",
        "PCW_UART1_UART1_IO"            : "MIO 48 .. 49",
        "PCW_PRESET_BANK1_VOLTAGE"      : "LVCMOS 1.8V",
        "PCW_USE_M_AXI_GP0"             : "1",
        "PCW_USE_S_AXI_GP0"             : "1",
        "PCW_USB0_PERIPHERAL_ENABLE"    : "1",
        "PCW_USB0_USB0_IO"              : "MIO 28 .. 39",
        "PCW_USB0_RESET_ENABLE"         : "1",
        "PCW_USB0_RESET_IO"             : "MIO 46",
        "PCW_EN_USB0"                   : "1"
    },
    "z7" : {
        "PCW_UIPARAM_DDR_PARTNO"        : "MT41K256M16 RE-125"
    },
    "original" : {
        "PCW_UIPARAM_DDR_PARTNO"        : "MT41K128M16 RE-125"
    }
}

# Platform -----------------------------------------------------------------------------------------

class Platform(Xilinx7SeriesPlatform):
    default_clk_name   = "clk125"
    default_clk_period = 1e9/125e6

    def __init__(self, variant="z7-20", toolchain="vivado"):
        device = {
            "z7-10": "xc7z010-clg400-1",
            "z7-20": "xc7z020-clg400-1",
            "original": "xc7z010-clg400-1"
        }[variant]
        ps7_config = ps7_config_variants["common"]
        ps7_config.update(ps7_config_variants["original" if variant == "original" else "z7"])

        Xilinx7SeriesPlatform.__init__(self, device, _io, _connectors, toolchain=toolchain)
        self.add_extension(_ps7_io)
        self.add_extension(_usb_uart_pmod_io)
        self.add_extension(_io_original if variant == "original" else _io_z7)
        self.add_connector(_connectors_original if variant == "original" else _connectors_z7)
        self.ps7_config = ps7_config

    def create_programmer(self):
        return VivadoProgrammer()

    def do_finalize(self, fragment):
        Xilinx7SeriesPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk125", loose=True), 1e9/125e6)



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Franck Jullien <franck.jullien@collshade.fr>
# Copyright (c) 2021 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from litex.build.generic_platform import *
from litex.build.efinix.platform import EfinixPlatform
from litex.build.efinix import EfinixProgrammer

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk
    ("clk40", 0, Pins("P19"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),
    ("clk50", 0, Pins("AA8"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),
    ("clk20", 0, Pins("AA9"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),
    ("clk74_25", 0, Pins("J9"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),

    # Leds
    ("user_led", 0, Pins("AB16"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),
    ("user_led", 1, Pins("AA16"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),
    ("user_led", 2, Pins("AA15"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),
    ("user_led", 3, Pins("Y15"),  IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),
    ("user_led", 4, Pins("Y16"),  IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),
    ("user_led", 5, Pins("W16"),  IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),
    ("user_led", 6, Pins("AD16"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),
    ("user_led", 7, Pins("AC16"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),

    # Buttons
    ("user_btn", 0, Pins("AD15"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("WEAK_PULLUP")),
    ("user_btn", 1, Pins("AC15"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("WEAK_PULLUP")),
    ("user_btn", 2, Pins("W15"),  IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("WEAK_PULLUP")),
    ("user_btn", 3, Pins("V15"),  IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("WEAK_PULLUP")),

    # Switches
    ("user_sw", 0, Pins("U16"),  IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("WEAK_PULLUP")),
    ("user_sw", 1, Pins("T16"),  IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("WEAK_PULLUP")),
    ("user_sw", 2, Pins("T15"),  IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("WEAK_PULLUP")),
    ("user_sw", 3, Pins("U15"),  IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("WEAK_PULLUP")),

    # SPIFlash
    ("spiflash", 0,
        Subsignal("cs_n", Pins("R23")),
        Subsignal("clk",  Pins("P22")),
        Subsignal("mosi", Pins("N24")),
        Subsignal("miso", Pins("N23")),
        #Subsignal("wp",   Pins("R19")),
        #Subsignal("hold", Pins("R17")),
        IOStandard("3.3_V_LVTTL_/_LVCMOS")
    ),

    ("spiflash4x", 0,
        Subsignal("cs_n", Pins("R23")),
        Subsignal("clk",  Pins("P22")),
        Subsignal("dq",   Pins("N24 N23 R19 R17")),
        IOStandard("3.3_V_LVTTL_/_LVCMOS")
    ),

    # RGMII Ethernet
    ("eth_clocks", 0,
        Subsignal("tx", Pins("L16")),
        Subsignal("rx", Pins("M23")),
        IOStandard("3.3_V_LVTTL_/_LVCMOS")
    ),

    ("eth", 0,
        Subsignal("rx_ctl",  Pins("F24")),
        Subsignal("rx_data", Pins("J20 K20 J19 H20")),
        Subsignal("tx_ctl",  Pins("G24")),
        Subsignal("tx_data", Pins("L23 L22 K21 K19")),
        Subsignal("rst_n",   Pins("F21")),
        Subsignal("mdc",     Pins("G21")),
        Subsignal("mdio",    Pins("G19")),
        IOStandard("3.3_V_LVTTL_/_LVCMOS")
    ),

    ("eth_clocks", 1,
        Subsignal("tx", Pins("M18")),
        Subsignal("rx", Pins("M24")),
        IOStandard("3.3_V_LVTTL_/_LVCMOS")
    ),

    ("eth", 1,
        Subsignal("rx_ctl",  Pins("D23")),
        Subsignal("rx_data", Pins("M8 L7 M7 N6")),
        Subsignal("tx_ctl",  Pins("E24")),
        Subsignal("tx_data", Pins("N7 N9 N8 N5")),
        Subsignal("rst_n",   Pins("E23")),
        Subsignal("mdc",     Pins("E22")),
        Subsignal("mdio",    Pins("D24")),
        IOStandard("3.3_V_LVTTL_/_LVCMOS")
    ),

    # DRAM.
    ("dram_pll_refclk", 0, Pins("AA8"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),

    # MIPI
    ("mipi_refclk", 0, Pins("G9"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),
]

# Bank voltage ---------------------------------------------------------------------------------------

_bank_info = [
            ("1A",          "3.3 V LVTTL / LVCMOS"),
            ("1B_1C",       "3.3 V LVTTL / LVCMOS"),
            ("1D_1E_1F_1G", "3.3 V LVTTL / LVCMOS"),
            ("2A",          "3.3 V LVTTL / LVCMOS"),
            ("2B",          "3.3 V LVTTL / LVCMOS"),
            ("2C",          "3.3 V LVTTL / LVCMOS"),
            ("2D",          "3.3 V LVTTL / LVCMOS"),
            ("2E",          "3.3 V LVTTL / LVCMOS"),
            ("2F",          "3.3 V LVTTL / LVCMOS"),
            ("3D_TR_BR",    "3.3 V LVTTL / LVCMOS"),
            ("4A",          "3.3 V LVTTL / LVCMOS"),
            ("4B",          "3.3 V LVTTL / LVCMOS"),
            ("4C",          "3.3 V LVTTL / LVCMOS"),
            ("4D",          "3.3 V LVTTL / LVCMOS"),
            ("4E",          "3.3 V LVTTL / LVCMOS"),
            ("4F",          "3.3 V LVTTL / LVCMOS"),
            ("BL",          "1.2 V"),
            ("TL",          "1.2 V"),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    ("pmod_a", " T12  V11  Y12  Y10  U12  U11  W12  Y11"),
    ("pmod_b", "AD11 AD10 AC10  W10 AA11 AC11 AA10  V10"),
    ("pmod_c", " A11  D11  J11  G10  B11  E11  H11  F10"),
    ("pmod_d", " D10  A10   A9   D9  E10  B10   B9   C9"),
    ("pmod_e", " C24  B23  A20  A22  B24  A23  B20  B22"),
    ("pmod_f", " V21  V20 AD20 AD19  U21  V19 AC20 AC19"),
]

# PMODS --------------------------------------------------------------------------------------------

def raw_pmod_io(pmod):
    return [(pmod, 0, Pins(" ".join([f"{pmod}:{i:d}" for i in range(8)])), IOStandard("3.3_V_LVTTL_/_LVCMOS"))]

def usb_pmod_io(pmod):
    return [
        # USB-UART PMOD: https://store.digilentinc.com/pmod-usbuart-usb-to-uart-interface/
        ("usb_uart", 0,
            Subsignal("tx", Pins(f"{pmod}:1")),
            Subsignal("rx", Pins(f"{pmod}:2")),
            IOStandard("3.3_V_LVTTL_/_LVCMOS")
        ),
    ]

# Platform -----------------------------------------------------------------------------------------

class Platform(EfinixPlatform):
    default_clk_name   = "clk40"
    default_clk_period = 1e9/40e6

    def __init__(self, toolchain="efinity"):
        EfinixPlatform.__init__(self, "T120F576I4", _io, _connectors, iobank_info=_bank_info, toolchain=toolchain)

    def create_programmer(self):
        return EfinixProgrammer()

    def do_finalize(self, fragment):
        EfinixPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk40", loose=True), 1e9/40e6)



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2020 David Corrigan <davidcorrigan714@gmail.com>
# Copyright (c) 2020 Alan Green <avg@google.com>
# SPDX-License-Identifier: BSD-2-Clause

from migen import *
from migen.genlib.resetsync import AsyncResetSynchronizer

from litex.gen import *

from litex_boards.platforms import lattice_crosslink_nx_evn

from litex.soc.cores.ram import NXLRAM
from litex.soc.cores.clock import NXPLL
from litex.build.io import CRG
from litex.build.generic_platform import *

from litex.soc.cores.clock import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.soc import SoCRegion
from litex.soc.integration.builder import *
from litex.soc.cores.led import LedChaser

kB = 1024
mB = 1024*kB


# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq):
        self.rst    = Signal()
        self.cd_por = ClockDomain()
        self.cd_sys = ClockDomain()

        # Built in OSC
        self.hf_clk = NXOSCA()
        hf_clk_freq = 25e6
        self.hf_clk.create_hf_clk(self.cd_por, hf_clk_freq)

        # Power on reset
        por_count = Signal(16, reset=2**16-1)
        por_done  = Signal()
        self.comb += por_done.eq(por_count == 0)
        self.sync.por += If(~por_done, por_count.eq(por_count - 1))

        self.rst_n = platform.request("gsrn")
        self.specials += AsyncResetSynchronizer(self.cd_por, ~self.rst_n)

        # PLL
        self.sys_pll = sys_pll = NXPLL(platform=platform, create_output_port_clocks=True)
        self.comb += sys_pll.reset.eq(self.rst)
        sys_pll.register_clkin(self.cd_por.clk, hf_clk_freq)
        sys_pll.create_clkout(self.cd_sys, sys_clk_freq)
        self.specials += AsyncResetSynchronizer(self.cd_sys, ~self.sys_pll.locked | ~por_done )


# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    mem_map = {
        "rom"      : 0x00000000,
        "sram"     : 0x40000000,
        "main_ram" : 0x60000000,
        "csr"      : 0xf0000000,
    }

    def __init__(self, sys_clk_freq=75e6, device="LIFCL-40-9BG400C", toolchain="radiant",
        with_led_chaser = True,
        with_spi_flash  = False,
        with_uartbone   = False,
        **kwargs):
        platform = lattice_crosslink_nx_evn.Platform(device=device, toolchain=toolchain)

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq)

        # SoCCore -----------------------------------------_----------------------------------------
        # Disable Integrated SRAM since we want to instantiate LRAM specifically for it
        kwargs["integrated_sram_size"] = 0
        # Make serial_pmods available
        platform.add_extension(lattice_crosslink_nx_evn.serial_pmods)
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on Crosslink-NX Evaluation Board", **kwargs)

        # 128KB LRAM (used as SRAM) ---------------------------------------------------------------
        self.spram = NXLRAM(32, 64*kB)
        self.bus.add_slave("sram", self.spram.bus, SoCRegion(origin=self.mem_map["sram"], size=16*kB))

        self.main_ram = NXLRAM(32, 64*kB)
        self.bus.add_slave("main_ram", self.main_ram.bus, SoCRegion(origin=self.mem_map["main_ram"], size=64*kB))

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = Cat(*[platform.request("user_led", i) for i in range(14)]),
                sys_clk_freq = sys_clk_freq)

        # UARTBone ---------------------------------------------------------------------------------
        if with_uartbone:
            self.add_uartbone()

        # SPI Flash --------------------------------------------------------------------------------
        if with_spi_flash:
            from litespi.modules import MX25L12833F
            from litespi.opcodes import SpiNorFlashOpCodes as Codes
            self.add_spi_flash(mode="4x", clk_freq=100_000, module=MX25L12833F(Codes.READ_4_4_4), with_master=True)


# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=lattice_crosslink_nx_evn.Platform, description="LiteX SoC on Crosslink-NX Eval Board.")
    parser.add_target_argument("--device",        default="LIFCL-40-9BG400C", help="FPGA device (LIFCL-40-9BG400C, LIFCL-40-8BG400CES, or LIFCL-40-8BG400CES2).")
    parser.add_target_argument("--sys-clk-freq",  default=75e6, type=float,   help="System clock frequency.")
    parser.add_target_argument("--serial",        default="serial",           help="UART Pins (serial (requires R15 and R17 to be soldered) or serial_pmod[0-2]).")
    parser.add_target_argument("--programmer",    default="radiant",          help="Programmer (radiant or ecpprog or openocd).")
    parser.add_target_argument("--address",       default=0x0,                help="Flash address to program bitstream at.")
    parser.add_target_argument("--prog-target",   default="direct",           help="Programming Target (direct or flash).")
    parser.add_target_argument("--with-spi-flash", action="store_true",       help="Enable SPI Flash (MMAPed).")
    parser.add_target_argument("--with-uartbone", action="store_true",        help="Add UartBone on 1st serial.")
    args = parser.parse_args()

    soc = BaseSoC(
        sys_clk_freq = args.sys_clk_freq,
        device       = args.device,
        toolchain    = args.toolchain,
        with_spi_flash = args.with_spi_flash,
        with_uartbone = args.with_uartbone,
        **parser.soc_argdict
    )
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer(args.prog_target, args.programmer)
        if args.programmer == "ecpprog" and args.prog_target == "flash":
            prog.flash(args.address, builder.get_bitstream_filename(mode="sram"))
        else:
            prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2023 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

# Build/Use ----------------------------------------------------------------------------------------
# Build/Load bitstream:
# ./ocp_tap_timecard.py --uart-name=crossover --with-pcie --with-smas --build --driver --load (or --flash)
#
#.Build the kernel and load it:
# cd build/<platform>/driver/kernel
# make
# sudo ./init.sh
#
# Test userspace utilities:
# cd build/<platform>/driver/user
# make
# ./litepcie_util info
# ./litepcie_util scratch_test
# ./litepcie_util dma_test
# ./litepcie_util uart_test

import os

from migen import *

from litex.gen import *

from litex_boards.platforms import ocp_tap_timecard

from litex.soc.interconnect.csr import *
from litex.soc.interconnect import stream
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *

from litex.soc.cores.clock import *
from litex.soc.cores.led import LedChaser
from litex.soc.cores.xadc import XADC
from litex.soc.cores.dna  import DNA

from litedram.modules import MT41K256M16
from litedram.phy import s7ddrphy

from litepcie.phy.s7pciephy import S7PCIEPHY
from litepcie.software import generate_litepcie_software

# CRG ----------------------------------------------------------------------------------------------

class CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq):
        self.rst          = Signal()
        self.cd_sys       = ClockDomain()
        self.cd_sys4x     = ClockDomain()
        self.cd_sys4x_dqs = ClockDomain()
        self.cd_idelay    = ClockDomain()

        # Clk/Rst
        clk200 = platform.request("clk200")

        # PLL
        self.pll = pll = S7PLL()
        self.comb += pll.reset.eq(self.rst)
        pll.register_clkin(clk200, 200e6)
        pll.create_clkout(self.cd_sys,       sys_clk_freq)
        pll.create_clkout(self.cd_sys4x,     4*sys_clk_freq)
        pll.create_clkout(self.cd_sys4x_dqs, 4*sys_clk_freq, phase=90)
        pll.create_clkout(self.cd_idelay,    200e6)
        platform.add_false_path_constraints(self.cd_sys.clk, pll.clkin) # Ignore sys_clk to pll.clkin path created by SoC's rst.

        self.idelayctrl = S7IDELAYCTRL(self.cd_idelay)

# BaseSoC -----------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, sys_clk_freq=100e6,
        with_led_chaser = True,
        with_pcie       = False,
        with_smas       = False,
        **kwargs):
        platform = ocp_tap_timecard.Platform()

        # CRG --------------------------------------------------------------------------------------
        self.crg = CRG(platform, sys_clk_freq)

        # SoCCore ----------------------------------------------------------------------------------
        kwargs["uart_name"] = "crossover"
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on OCP-TAP TimeCard", **kwargs)

        # XADC -------------------------------------------------------------------------------------
        self.xadc = XADC()

        # DNA --------------------------------------------------------------------------------------
        self.dna = DNA()
        self.dna.add_timing_constraints(platform, sys_clk_freq, self.crg.cd_sys.clk)

        # Leds -------------------------------------------------------------------------------------
        if with_led_chaser:
            self.leds = LedChaser(
                pads         = platform.request_all("user_led"),
                sys_clk_freq = sys_clk_freq
            )

        # PCIe -------------------------------------------------------------------------------------
        if with_pcie:
            self.pcie_phy = S7PCIEPHY(platform, platform.request("pcie_x1"),
                data_width = 64,
                bar0_size  = 0x20000)
            self.add_pcie(phy=self.pcie_phy, ndmas=1, address_width=64)
            # FIXME: Apply it to all targets (integrate it in LitePCIe?).
            platform.add_period_constraint(self.crg.cd_sys.clk, 1e9/sys_clk_freq)
            platform.toolchain.pre_placement_commands.append("reset_property LOC [get_cells -hierarchical -filter {{NAME=~*gtp_channel.gtpe2_channel_i}}]")
            platform.toolchain.pre_placement_commands.append("set_property LOC GTPE2_CHANNEL_X0Y5 [get_cells -hierarchical -filter {{NAME=~*gtp_channel.gtpe2_channel_i}}]")

            # ICAP (For FPGA reload over PCIe).
            from litex.soc.cores.icap import ICAP
            self.icap = ICAP()
            self.icap.add_reload()
            self.icap.add_timing_constraints(platform, sys_clk_freq, self.crg.cd_sys.clk)

            # Flash (For SPIFlash update over PCIe).
            from litex.soc.cores.gpio import GPIOOut
            from litex.soc.cores.spi_flash import S7SPIFlash
            self.flash_cs_n = GPIOOut(platform.request("flash_cs_n"))
            self.flash      = S7SPIFlash(platform.request("flash"), sys_clk_freq, 25e6)

        # SMAs -------------------------------------------------------------------------------------
        if with_smas:
            # TODO:
            # - Use IO primitives to increase IO freqs (up to 800Mz would be fine) and PCIe Gen2 X1
            #   should be able to saturate it (~3.4Gbps of useful bandwidth).
            # - Allow IO freq configuration (through PLL/MMCM and/or dividers).

            class SMAIOs(LiteXModule):
                def __init__(self, pcie_data_width=64, io_data_width=4):
                    # Endpoints.
                    self.sink   = stream.Endpoint([("data", pcie_data_width)])
                    self.source = stream.Endpoint([("data", pcie_data_width)])

                    # CSRs.
                    self.control = CSRStorage(fields=[
                        CSRField("in_en",  size=io_data_width, description="Input  enable control (1bit per SMA)."),
                        CSRField("out_en", size=io_data_width, description="Output enable control (1bit per SMA)."),
                    ])
                    self.output  = CSRStorage(io_data_width, description="SMA Reg Output (1bit per SMA).")
                    self.input   = CSRStatus(io_data_width,  description="SMA Reg Input  (1bit per SMA).")

                    # # #

                    # SMA Pads.
                    sma_pads = [platform.request("sma", i) for i in range(io_data_width)]

                    # SMA Buffer Control.
                    for i in range(io_data_width):
                        self.sync += sma_pads[i].dat_in_en.eq( self.control.fields.in_en)
                        self.sync += sma_pads[i].dat_out_en.eq(self.control.fields.in_en)

                    # SMA TX Reg, allow direct (and slow...) control of SMA IOs.
                    for i in range(io_data_width):
                        self.sync += If(self.output.storage[i],
                            sma_pads[i].dat_out.eq(1)
                        )

                    # SMA TX Data Pipeline.
                    self.tx_converter = tx_converter = stream.Converter(pcie_data_width, io_data_width)
                    self.comb += self.sink.connect(self.tx_converter.sink)
                    self.comb += self.tx_converter.source.ready.eq(1)
                    for i in range(io_data_width):
                        self.sync += If(tx_converter.source.valid & tx_converter.source.data[i],
                            sma_pads[i].dat_out.eq(1)
                        )

                    # SMA RX Reg, allow direct (and slow...) visualization of SMA IOs.
                    for i in range(io_data_width):
                        self.sync += self.input.status[i].eq(sma_pads[i].dat_in)

                    # SMA RX Data Pipeline.
                    self.rx_converter = stream.Converter(io_data_width, pcie_data_width)
                    self.comb += self.rx_converter.sink.valid.eq(1)
                    for i in range(io_data_width):
                        self.sync += self.rx_converter.sink.data[i].eq(sma_pads[i].dat_in)
                    self.comb += self.rx_converter.source.connect(self.source)

            self.smas = SMAIOs(pcie_data_width=64, io_data_width=4)
            self.comb += self.pcie_dma0.source.connect(self.smas.sink)
            self.comb += self.smas.source.connect(self.pcie_dma0.sink)

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=ocp_tap_timecard.Platform, description="LiteX SoC on OCP-TAP TimeCard.")
    parser.add_target_argument("--flash",        action="store_true",       help="Flash bitstream.")
    parser.add_target_argument("--sys-clk-freq", default=100e6, type=float, help="System clock frequency.")
    parser.add_target_argument("--with-pcie",    action="store_true", help="Enable PCIe support.")
    parser.add_target_argument("--with-smas",    action="store_true", help="Enable SMAs support.")
    parser.add_target_argument("--driver",       action="store_true", help="Generate PCIe driver.")
    args = parser.parse_args()

    soc = BaseSoC(
        sys_clk_freq = args.sys_clk_freq,
        with_pcie    = args.with_pcie,
        with_smas    = args.with_smas,
        **parser.soc_argdict
    )

    builder  = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.driver:
        generate_litepcie_software(soc, os.path.join(builder.output_dir, "driver"))

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

    if args.flash:
        prog = soc.platform.create_programmer()
        prog.flash(0, builder.get_bitstream_filename(mode="flash"))

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Florent Kermarrec <florent@enjoy-digital.fr>
# SPDX-License-Identifier: BSD-2-Clause

from migen import *

from litex.gen import *

from litex_boards.platforms import litex_acorn_baseboard

from litex.soc.cores.clock import *
from litex.soc.integration.soc_core import *
from litex.soc.integration.builder import *
from litex.soc.cores.video import VideoHDMIPHY
from litex.soc.cores.bitbang import I2CMaster

from liteeth.phy.ecp5rgmii import LiteEthPHYRGMII

# CRG ----------------------------------------------------------------------------------------------

class _CRG(LiteXModule):
    def __init__(self, platform, sys_clk_freq, with_video_pll=False):
        self.rst    = Signal()
        self.cd_por = ClockDomain()
        self.cd_sys = ClockDomain()

        # # #

        # Clk / Rst
        clk50 = platform.request("clk50")

        # Power on reset
        por_count = Signal(16, reset=2**16-1)
        por_done  = Signal()
        self.comb += self.cd_por.clk.eq(clk50)
        self.comb += por_done.eq(por_count == 0)
        self.sync.por += If(~por_done, por_count.eq(por_count - 1))

        # PLL
        self.pll = pll = ECP5PLL()
        self.comb += pll.reset.eq(~por_done | self.rst)
        pll.register_clkin(clk50, 50e6)
        pll.create_clkout(self.cd_sys, sys_clk_freq)

        # Video PLL
        if with_video_pll:
            self.video_pll = video_pll = ECP5PLL()
            self.comb += pll.reset.eq(~por_done | self.rst)
            video_pll.register_clkin(clk50, 50e6)
            self.cd_hdmi   = ClockDomain()
            self.cd_hdmi5x = ClockDomain()
            video_pll.create_clkout(self.cd_hdmi,    40e6, margin=0)
            video_pll.create_clkout(self.cd_hdmi5x, 200e6, margin=0)

# BaseSoC ------------------------------------------------------------------------------------------

class BaseSoC(SoCCore):
    def __init__(self, sys_clk_freq=75e6, toolchain="trellis",
        with_spi_flash      = False,
        with_ethernet       = False,
        with_etherbone      = False,
        with_video_terminal = False,
        with_lcd            = False,
        with_ws2812         = False,
        **kwargs):
        platform = litex_acorn_baseboard.Platform(toolchain=toolchain)

        # CRG --------------------------------------------------------------------------------------
        self.crg = _CRG(platform, sys_clk_freq, with_video_pll=with_video_terminal)

        # SoCCore ----------------------------------------------------------------------------------
        SoCCore.__init__(self, platform, sys_clk_freq, ident="LiteX SoC on LiteX M2 Baseboard", **kwargs)

        # SPI Flash --------------------------------------------------------------------------------
        if with_spi_flash:
            from litespi.modules import W25Q128JV
            from litespi.opcodes import SpiNorFlashOpCodes as Codes
            self.add_spi_flash(mode="4x", module=W25Q128JV(Codes.READ_1_1_4), with_master=True)

        # Ethernet / Etherbone ---------------------------------------------------------------------
        if with_ethernet or with_etherbone:
            self.ethphy = LiteEthPHYRGMII(
                clock_pads = self.platform.request("eth_clocks"),
                pads       = self.platform.request("eth"),
                rx_delay   = 0e-9)
            if with_ethernet:
                self.add_ethernet(phy=self.ethphy)
            if with_etherbone:
                self.add_etherbone(phy=self.ethphy)

        # Video ------------------------------------------------------------------------------------
        if with_video_terminal:
            self.videophy = VideoHDMIPHY(platform.request("hdmi"), clock_domain="hdmi", pn_swap=["g", "r"])
            self.add_video_terminal(phy=self.videophy, timings="800x600@60Hz", clock_domain="hdmi")

        # LCD --------------------------------------------------------------------------------------
        if with_lcd:
            self.i2c = I2CMaster(platform.request("lcd"))

        # M2 --------------------------------------------------------------------------------------
        self.comb += platform.request("m2_devslp").eq(0) # Enable SATA M2.

        # WS2812 ----------------------------------------------------------------------------------
        if with_ws2812:
            from litex.build.generic_platform import Pins, IOStandard
            from litex.soc.integration.soc import SoCRegion
            from litex.soc.cores.led import WS2812
            platform.add_extension([("ws2812", 0, Pins("pmod1:0"), IOStandard("LVCMOS33"))])
            self.ws2812 = WS2812(platform.request("ws2812"), nleds=64, sys_clk_freq=sys_clk_freq)
            self.bus.add_slave(name="ws2812", slave=self.ws2812.bus, region=SoCRegion(
                origin = 0x2000_0000,
                size   = 64*4,
            ))

# Build --------------------------------------------------------------------------------------------

def main():
    from litex.build.parser import LiteXArgumentParser
    parser = LiteXArgumentParser(platform=litex_acorn_baseboard.Platform, description="LiteX SoC on LiteX Acorn Baseboard.")
    parser.add_target_argument("--flash",        action="store_true",      help="Flash bitstream to SPI Flash.")
    parser.add_target_argument("--sys-clk-freq", default=75e6, type=float, help="System clock frequency.")
    ethopts = parser.target_group.add_mutually_exclusive_group()
    ethopts.add_argument("--with-ethernet",  action="store_true", help="Enable Ethernet support.")
    ethopts.add_argument("--with-etherbone", action="store_true", help="Enable Etherbone support.")
    sdopts = parser.target_group.add_mutually_exclusive_group()
    sdopts.add_argument("--with-spi-sdcard", action="store_true", help="Enable SPI-mode SDCard support.")
    sdopts.add_argument("--with-sdcard",     action="store_true", help="Enable SDCard support.")
    viopts = parser.target_group.add_mutually_exclusive_group()
    viopts.add_argument("--with-video-terminal", action="store_true", help="Enable Video Terminal (HDMI).")
    parser.add_target_argument("--with-spi-flash", action="store_true",      help="Enable SPI Flash (MMAPed).")
    parser.add_target_argument("--with-lcd",       action="store_true",      help="Enable OLED LCD support.")
    parser.add_target_argument("--with-ws2812",    action="store_true",      help="Enable WS2812 on PMOD1:0.")

    args = parser.parse_args()

    soc = BaseSoC(
        sys_clk_freq        = args.sys_clk_freq,
        toolchain           = args.toolchain,
        with_spi_flash      = args.with_spi_flash,
        with_ethernet       = args.with_ethernet,
        with_etherbone      = args.with_etherbone,
        with_video_terminal = args.with_video_terminal,
        with_lcd            = args.with_lcd,
        with_ws2812         = args.with_ws2812,
        **parser.soc_argdict
    )
    if args.with_spi_sdcard:
        soc.add_spi_sdcard()
    if args.with_sdcard:
        soc.add_sdcard()
    builder = Builder(soc, **parser.builder_argdict)
    if args.build:
        builder.build(**parser.toolchain_argdict)

    if args.load:
        prog = soc.platform.create_programmer()
        prog.load_bitstream(builder.get_bitstream_filename(mode="sram"))

    if args.flash:
        prog = soc.platform.create_programmer()
        prog.flash(None, prog.load_bitstream(builder.get_bitstream_filename(mode="flash")))

if __name__ == "__main__":
    main()



#
# This file is part of LiteX-Boards.
#
# Copyright (c) 2021 Andrew Dennison <andrew@motec.com.au>
# Copyright (c) 2022 Charles-Henri Mousset <ch.mousset@gmail.com>
# SPDX-License-Identifier: BSD-2-Clause

import os
from litex.build.generic_platform import *
from litex.build.efinix.platform import EfinixPlatform
from litex.build.efinix.programmer import EfinixAtmelProgrammer

# IOs ----------------------------------------------------------------------------------------------

_io = [
    # Clk
    ("clk33", 0, Pins("C3"), IOStandard("3.3_V_LVTTL_/_LVCMOS")),  # net PLL_IN

    # Buttons
    ("user_btn", 0, Pins("G1"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("WEAK_PULLUP")),
    ("user_btn", 1, Pins("F1"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("WEAK_PULLUP")),

    # Leds
    ("user_led", 0, Pins("G4"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),
    ("user_led", 1, Pins("J2"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),
    ("user_led", 2, Pins("C2"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),
    ("user_led", 3, Pins("E3"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),
    ("user_led", 4, Pins("B3"), IOStandard("3.3_V_LVTTL_/_LVCMOS"), Misc("DRIVE_STRENGTH=3")),

    # SPIFlash (W25Q80DV)
    ("spiflash", 0,
        Subsignal("cs_n", Pins("J4")),  # net SPI_SS
        Subsignal("clk",  Pins("H4")),  # net SPI_SCLK
        Subsignal("mosi", Pins("F4")),  # net SPI_MOSI
        Subsignal("miso", Pins("H3")),  # net SPI_MISO
        #Subsignal("wp",   Pins("")),
        #Subsignal("hold", Pins("")),
        IOStandard("3.3_V_LVTTL_/_LVCMOS")
     ),
]

# Connectors ---------------------------------------------------------------------------------------

_connectors = [
    # use "j3:1" reference for pin 1 of J3. This makes it a 1:1 translation with connector numbering
    ("j3", "ZERO #N/A #N/A G5 F1 G4 E1 J3 C2 G3 D2 J2 E3 H2 D3 F3 C3 G1 B3 #N/A #N/A A2 A2"),
    ("j4", "ZERO #N/A #N/A A5 B8 B5 C8 C5 D6 A6 B9 B6 C9 C6 D7 C7 D8 A8 D9 A9 E8 A2 A2"),
    ("j5", "ZERO #N/A #N/A F8 J9 E7 J8 F7 G8 E6 H8 F6 J7 F5 G6 G9 H6 H9 J6 #N/A #N/A A2 A2"),
]

# Platform -----------------------------------------------------------------------------------------

class Platform(EfinixPlatform):
    default_clk_name = "clk33"
    default_clk_period = 1e9/33.333e6

    def __init__(self, toolchain="efinity"):
        EfinixPlatform.__init__(self, "T8F81C2", _io, _connectors, toolchain=toolchain)

    def create_programmer(self):
        return EfinixAtmelProgrammer()

    def do_finalize(self, fragment):
        EfinixPlatform.do_finalize(self, fragment)
        self.add_period_constraint(self.lookup_request("clk33", loose=True), 1e9/33.333e6)
